<?php

namespace Teknisa\DF\Services;

/* Bibliotecas */
use Doctrine\ORM\EntityManager;
use Fiscal\Entities\Notafiscal;
use Geral\Entities\Empresa;
use Geral\Entities\Endefili;
use Geral\Entities\Inscrestad;
use Geral\Entities\Paramempresa;
use Geral\Entities\Tipooperac;
use Html2Text\Html2Text;
use Indeterminado\Entities\Cartacornfe;
use Indeterminado\Entities\Relacnflotenfe;
use NFePHP\Common\Certificate;
use NFePHP\Common\Configure\Configure;
use NFePHP\Common\Exception\RuntimeException;
use NFePHP\Common\Files\FilesFolders;
use NFePHP\Common\Signer;
use NFePHP\Common\Soap\SoapCurl;
use NFePHP\Common\UFList;
use NFePHP\Mail\Mail;
use NFePHP\NFe\Common\Config;
use NFePHP\NFe\Tools as NFe;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use \Teknisa\Libs\Util\SMTP;
use Teknisa\Model\Fiscal\Entities\Arqxmlnfe;
use Teknisa\Model\Fiscal\Entities\Lotenfe;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

/* Classes DF */
use Teknisa\DF\Factories\ProceduresNfFactory;
use Teknisa\DF\Factories\TransmissaoNfeFactory;
use Teknisa\DF\Services\AprovacaoLancamentosService;
use Teknisa\DF\Services\NFe\Dom;
use Teknisa\DF\Services\NFe\Danfe;
use Teknisa\DF\Services\NFe\ReturnNFe;
use Teknisa\DF\Services\ValidacoesEstoqueService;
use Teknisa\DF\Services\XmlNFeService;

class TransmissaoNfeService {
    protected $transmissaoNfeFactory;
    protected $xmlNFeService;
    protected $proceduresNfFactory;
    protected $novoCodigo;
    protected $entityManager;
    protected $environment;
    protected $validacoesEstoqueService;
    protected $validacoesTransmissaoNfeService;
    protected $aprovacaoLancamentosService;
    protected $masks;
    protected $util;
    protected $corpoNF;
    protected $tpEmis;
    protected $xml;
    protected $aXml = array();
    protected $nota;
    protected $aNota = array();
    protected $aResposta = [];
    protected $aRespostaContingency = [];
    protected $notasLoteSuccess = [];
    protected $errosAprovacao = [];
    protected $sucessoAprovacao = [];
    protected $errosEmail = [];
    protected $sucessoEmail = [];
    protected $idLote;
    protected $template;
    protected $msgHtml;
    protected $msgTxt;

    protected $assDir = "";
    protected $canDir = "";
    protected $entDir = "";
    protected $envDir = "";
    protected $envaprDir = "";
    protected $envrejDir = "";
    protected $envdenDir = "";
    protected $rejDir = "";
    protected $valDir = "";
    protected $tempDir = "";
    protected $pdfDir = "";
    /** @var NFe $nfeTools */
    protected $nfeTools;
    
    private $fields;
    private $subject;

    private $numeroNotaFiscal;
    private $rowCupomFiscal;

    public $aRetorno = array();

    const AMBIENTE_PRODUCAO = '1';
    const AMBIENTE_HOMOLOGACAO = '2';

    const BLOQUEADO = 'S';
    const DESBLOQUADO = 'N';
    const CONTADOR_NOVOCODIGO_NRLOTE = 'LOTENFE/FILIAL';
    const CONTADOR_NOVOCODIGO_LOTECCE = 'LOTECCE';
    const TAMANHO_LOTENFE = 15;

    const ID_LOTE_PROCESSADO = 104;
    const ID_LOTE_EM_PROCESSAMENTO = '105';
    const ID_LOTE_NAO_LOCALIZADO = '106';
    const ID_LOTE_EVENTO_PROCESSADO = 128;

    const TIPO_EMISSAO_NORMAL = 1;
    const TIPO_EMISSAO_SCAN   = 3;
    const TIPO_EMISSAO_EPEC   = 4;
    const TIPO_EMISSAO_SVAN_AN   = 6;
    const TIPO_EMISSAO_SVAN_RS   = 7;

    const SERVICO_PARALISADO_TEMPORARIAMENTE = '108';
    const SERVICO_PARALISADO_SEM_PREVISAO = '109';
    const SERVICO_ATIVO = '107';

    const EVENTO_CANCELAMENTO_REG = '110111';
    const EVENTO_EPEC_AUTORIZADO = 124;
    const EVENTO_REGISTRADO_N_VINCULADO = 136;

    const PROBLEMA_RECIBO_OU_CNPJ = 'C';
    const LOTE_NAO_LOCALIZADO = 'P';
    const LOTE_PROCESSADO = 'C';
    const LOTE_EM_PROCESSAMENTO = 'E';
    const LOTE_TRANSMITIDO = 'T';

    const ID_NF_AUTORIZADA = 100;
    const ID_NF_AUTORIZADA_FORA_DO_PRAZO = 150;
    const ID_NF_DENEGADA = 110;
    const ID_NF_DENEGADA_EMIT = 301;
    const ID_NF_DENEGADA_DEST = 302;
    const ID_NF_DUPLICIDADE = 204;
    const ID_NF_DUPLICIDADE_CHAVE_ACESSO_MAIS_DE_1X = 344;
    const ID_NF_DUPLICIDADE_DIFERENCA_CHAVE_ACESSO = 539;

    const STATUS_NF_TRANSMITIDA = 'T';
    const STATUS_NF_ACEITA = 'A';
    const STATUS_NF_REJEITADA = 'R';
    const STATUS_NF_DENEGADA = 'D';
    const STATUS_NF_PENDENTE = 'P';

    const TIPO_ARQ_XML_DADOS_NOTA_FISCAL = 'D';
    const TIPO_ARQ_XML_AUTORIZACAO = 'A';
    const TIPO_ARQ_XML_CANCELAMENTO = 'C';
    const TIPO_ARQ_XML_CARTA_CORRECAO = 'E';

    const TIPO_END_PRINCIPAL = 'P';

    const DOWNLOAD = 'D';
    const VISUALIZAR = 'V';

    const FILESERVER_LINK   = 'http://midia.teknisa.com/files/';

    public function __construct(TransmissaoNfeFactory $transmissaoNfeFactory,
                                NewCode $novoCodigo,
                                EntityManager $entityManager,
                                ValidacoesEstoqueService $validacoesEstoqueService,
                                ProceduresNfFactory $proceduresNfFactory,
                                ValidacoesTransmissaoNfeService $validacoesTransmissaoNfeService,
                                XmlNFeService $xmlNFeService,
                                Utilities $util,
                                Environment $environment,
                                AprovacaoLancamentosService $aprovacaoLancamentosService,
                                SMTP $smtpService,
                                $crudService,
                                $fileServerLink)
    {
        $this->transmissaoNfeFactory = $transmissaoNfeFactory;
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->aprovacaoLancamentosService = $aprovacaoLancamentosService;
        $this->proceduresNfFactory = $proceduresNfFactory;
        $this->validacoesTransmissaoNfeService = $validacoesTransmissaoNfeService;
        $this->xmlNFeService = $xmlNFeService;
        $this->masks = new Masks();
        $this->util = $util;
        $this->environment = $environment;
        $this->pdfDir = sys_get_temp_dir();
        $this->smtpService = $smtpService;
        $this->fileServerLink = $fileServerLink;
        
        $this->fields = new \stdClass();
        $this->fields->destinatario = '';
        $this->fields->data = '';
        $this->fields->numero = '';
        $this->fields->valor = 0;
        $this->fields->chave = '';
        $this->fields->data = '';
        $this->fields->correcao = '';
        $this->fields->conduso = '';
        $this->fields->chave   = '';
        
        if($this->environment->getNrOrgTrab() == 1) {
            $this->SAAS = false;
        }
        else {
            $this->SAAS = true;
        }
    }
    
    /**
     * Html Templates
     * @var array
     */
    protected $templates = [
        'NFe'=> "<p>E-mail referente a emissão da Nota Fiscal Eletrônica nº {numero}.</p>".
                "Chave de Acesso: {chave}".
                
                "<p>Emitente: {emitente}<br>Destinatário: {destinatario}</p>",
                
        'CCe'=> "<p>E-mail referente a Carta de Correção para a Nota Fiscal Eletrônica nº {numero}.</p>".
                "Chave de Acesso: {chave}".
                
                "<p>Emitente: {emitente}<br>Destinatário: {destinatario}</p>",
                
        'C' => "<p>E-mail referente ao Cancelamento para a Nota Fiscal Eletrônica nº {numero}.</p>".
                "Chave de Acesso: {chave}".
                
                "<p>Emitente: {emitente}<br>Destinatário: {destinatario}</p>",
    ];

    public function montaConfigTransmissao($empresa, $inscricao, $filial, $tpEndEmissao) {
        
        $organizacao = $this->environment->getNrOrgTrab(); 
        $objEmpresa = Empresa::findOneByCdempresaAndNrorg($empresa, $organizacao);
        $cnpj = $this->masks->removeMascaraInscricao($objEmpresa->getNrinsjurempr());

        $razaosocial = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($objEmpresa->getNmrazsocempr())));
        $nomefantasia = $razaosocial;
        $dadosProcessFilial = $this->validacoesEstoqueService->retornaDadosProcessFilial($filial, $organizacao);

        if(strpos("ISENTO",strtoupper($inscricao)) || strpos("ISENTA", strtoupper($inscricao))){
            $ie = "ISENTO";
        }
        else{
            $ie = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($inscricao)));
        }
        
        /** @var Inscrestad $inscrestad */
        $inscrestad = self::getInscricaoEstadual($empresa, $organizacao, $inscricao);
        
        // 1-Produção 2-Homologação
        $tpAmb = (int) $inscrestad->getIdtpambientenfe();
        
        $endefili = Endefili::findOneByCdfilialAndIdtpendefiliAndNrorg(
            $filial,
            $tpEndEmissao,
            $organizacao
        );
        $siglaUF = $endefili->getSgestado();
        
        $smtpParams = $this->getInfoSMTP($filial, $organizacao);
        $mailAuth = true;
        $mailFromName = $objEmpresa->getNmrazsocempr();
        $mailFromMail = self::getEmailRemetente($filial, $organizacao);
        $mailReplayToMail = self::getEmailRemetente($filial, $organizacao);

        //$mailFrom = 'no-reply@teknisa.com';
        //$mailSmtp = 'smtp.teknisa.com';
        //$mailUser = 'app@tecfood.com';
        //$mailPass = '@AppTecFood2015';
        //$mailProtocol = '';
        //$mailPort = '25';
        //$mailFrom = $mailFrom ? : $mailFromMail;
        //$mailPort = '587';
        
         /** @var Paramempresa $paramempresa */
        $paramempresa = Paramempresa::findOneByCdempresaAndNrorg($empresa, $organizacao);
    
        if($paramempresa) {
            $aprovacaoAutomatica = $paramempresa->getIdaprovnftrans();
            $regime = $paramempresa->getCdsitucrt();
            $emailAutomatico     = $this->transmissaoNfeFactory->utilizaEmailTransmissao($empresa)[0]["IDEMAILNFTRANS"];
        }
        else {
            throw new \Exception("Parametros da empresa $empresa não encontrados.");
        }

        $dadosProcessFilial = $this->validacoesEstoqueService->retornaDadosProcessFilial($filial, $organizacao);
        $objEmpresa = Empresa::findOneByCdempresaAndNrorg($empresa, $organizacao);
        
        $aMailConf = array(
            'from'      => $mailFromMail,
            'fantasy'   => $mailFromName,
            'replyTo'   => $mailReplayToMail,
            'replyName' => $mailFromName
        );
        
        $aMailConf = array_merge($smtpParams,$aMailConf);
        $aMailConf = $this->retirarArrobaEmail($aMailConf);
        
        $config = [
            //"atualizacao" => "2017-02-20 09:11:21",
            "tpAmb"     => $tpAmb,
            "razaosocial" => $nomefantasia,
            "nomefantasia" => $nomefantasia,
            "siglaUF"   => $siglaUF,
            "cnpj"      => $cnpj,
            "schemes"   => "PL_009_V4",
            "versao"    => "4.00",
            "ie"        => $ie,
            "aMailConf" => $aMailConf,
            "regime"    => $regime,
            "cdempresa" => $empresa,
            "nrorg"     => $this->environment->getNrOrgTrab(),
            "iest"      => '',
            "im"        => '',
            'cnae'      => '',
            'dadosFilial' => $dadosProcessFilial,
            'aprovacaoAutomatica' => $aprovacaoAutomatica,
            'emailAutomatico'     => $emailAutomatico
            // "tokenIBPT" => "AAAAAAA",
            // "CSC" => "GPB0JBWLUR6HWFTVEAS6RJ69GPCROFPBBB8G",
            // "CSCid" => "000001",
            //"aProxyConf" => [
            //    "proxyIp" => "",
            //    "proxyPort" => "",
            //    "proxyUser" => "",
            //    "proxyPass" => ""
            //]
        ];
       
        $resp = Config::validate(json_encode($config));
        return $resp;
    }
    
    private function validaDiretoriosTransmissao($organizacao, $inscricao){
        $raiz = realpath(__DIR__ . "/../../../../../../../../") . DIRECTORY_SEPARATOR . "anexos" . DIRECTORY_SEPARATOR . "transmissaoNFe";
        $nrorgDir = $raiz . DIRECTORY_SEPARATOR . $organizacao;
        $ieDir    = $nrorgDir . DIRECTORY_SEPARATOR . $inscricao;

        $xmlDir = $this->validaDiretorioXML($ieDir);
        $certDir = $this->validaDiretorioCert($ieDir);
        $downloadDir = $this->validaDiretorioDownload($ieDir);

        return array('xmlDir' => $xmlDir, 'certDir' => $certDir, 'downloadDir' => $downloadDir);
    }

    /**
     * @param $row
     * @return string
     * @throws \Exception
     */
    private function validaDiretorioXML($ieDir)
    {
        $xmlDir = $ieDir . DIRECTORY_SEPARATOR . "xmls";
        
        if (!is_dir($xmlDir)) {
            if(!is_dir($ieDir)) {
                if (!mkdir($ieDir, 0777, true)) {
                    throw new \Exception(
                        "Não foi possivel criar o diretorio $ieDir. Verifique as permissões"
                    );
                }
            }
            if (!mkdir($xmlDir, 0777, true)) {
                throw new \Exception(
                    "Não foi possivel criar o diretorio $xmlDir. Verifique as permissões"
                );
            }
        }

        return $xmlDir;
    }

    /**
     * @param $row
     * @return string
     * @throws \Exception
     */
    private function validaDiretorioCert($ieDir)
    {
        $certDir = $ieDir . DIRECTORY_SEPARATOR . "cert";

        if (!is_dir($certDir)) {
            if(!is_dir($ieDir)) {
                if (!mkdir($ieDir, 0777, true)) {
                    throw new \Exception(
                        "Não foi possivel criar o diretorio $ieDir. Verifique as permissões"
                    );
                }
            }
            if (!mkdir($certDir, 0777, true)) {
                throw new \Exception(
                    "Não foi possivel criar o diretorio $certDir. Verifique as permissões"
                );
            }
        }

        return $certDir;
    }

    /**
     * @param $row
     * @return string
     * @throws \Exception
     */
    private function validaDiretorioDownload($ieDir)
    {
        $downloadDir = $ieDir . DIRECTORY_SEPARATOR . "download" . DIRECTORY_SEPARATOR . $this->environment->getCdOperador();
        $downloadDanfeDir = $downloadDir . DIRECTORY_SEPARATOR . "danfe" . DIRECTORY_SEPARATOR;
        $downloadXMLDir = $downloadDir . DIRECTORY_SEPARATOR . "xml" . DIRECTORY_SEPARATOR;

        if (!is_dir($downloadDanfeDir)) {
            if (!mkdir($downloadDanfeDir, 0777, true)) {
                throw new \Exception(
                    "Não foi possivel criar o diretorio $downloadDanfeDir. Verifique as permissões"
                );
            }
        }

        if (!is_dir($downloadXMLDir)) {
            if (!mkdir($downloadXMLDir, 0777, true)) {
                throw new \Exception(
                    "Não foi possivel criar o diretorio $downloadXMLDir. Verifique as permissões"
                );
            }
        }

        return $downloadDir;
    }

    /**
     * Retorna inscricao estadual da empresa
     * @param $empresa
     * @param $cdinscestad
     * @param $organizacao
     * @return Inscrestad
     * @throws \Exception
     */
    private function getInscricaoEstadual($empresa, $organizacao, $inscricao)
    {
        $inscrestad = Inscrestad::findOneByCdempresaAndNrorgAndCdinscesta($empresa, $organizacao, $inscricao);
        if (!($inscrestad instanceof Inscrestad)) {
            throw new \Exception('Inscrição Estadual não Encontrada');
        }

        return $inscrestad;
    }

    /*private function inicializaVarDiretorios(){
        $ambiente = $this->nfeTools->config->tpAmb == '1' ? 'producao' : 'homologacao';
        $pathNFeFilesAmbiente = $this->nfeTools->config->pathNFeFiles . DIRECTORY_SEPARATOR . $ambiente;
        $this->assDir = $pathNFeFilesAmbiente . DIRECTORY_SEPARATOR . "assinadas";
        $this->canDir = $pathNFeFilesAmbiente . DIRECTORY_SEPARATOR . "canceladas";
        $this->entDir = $pathNFeFilesAmbiente . DIRECTORY_SEPARATOR . "entradas";
        $this->envDir = $pathNFeFilesAmbiente . DIRECTORY_SEPARATOR . "enviadas";
        $this->envaprDir = $this->envDir . DIRECTORY_SEPARATOR . "aprovadas";
        $this->envrejDir = $this->envDir . DIRECTORY_SEPARATOR . "rejeitadas";
        $this->envdenDir = $this->envDir . DIRECTORY_SEPARATOR . "denegadas";
        $this->rejDir = $pathNFeFilesAmbiente . DIRECTORY_SEPARATOR . "rejeitadas";
        $this->valDir = $pathNFeFilesAmbiente . DIRECTORY_SEPARATOR . "validadas";
        $this->tempDir = $pathNFeFilesAmbiente . DIRECTORY_SEPARATOR . "temporarias";
        
        $this->cceDir = $pathNFeFilesAmbiente . DIRECTORY_SEPARATOR . "cartacorrecao";

        $pathDownloadDir = $this->nfeTools->config->pathDownloadDir;
        $this->downloadDanfeDir = $pathDownloadDir . DIRECTORY_SEPARATOR .  "danfe";
        $this->downloadXMLDir = $pathDownloadDir . DIRECTORY_SEPARATOR .  "xml";
    }*/

    /*private function salvaXmlNfe($dir, $chave, $xml, $caminho = '', $tipo = ''){
        
        $dir = '/tmp/';
        if($tipo == 'C' || $tipo == 'CT'){
            $filename = $dir . DIRECTORY_SEPARATOR . $chave . 'procCanc.xml';
        }
        else{
            $filename = $dir . DIRECTORY_SEPARATOR . $chave . '-nfe.xml';
        }

        if(!$xml){
            return copy($caminho, $filename);
        }

        file_put_contents($filename, $xml);
        chmod($filename, 0777);
    }*/

    private function unlinkXMLs($nota){
        $verificaNotaRejeitada = $this->envrejDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml";
        if(is_file($verificaNotaRejeitada))
            unlink($this->envrejDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
        unlink($this->valDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
    }

    private function aprovacaoAutomatica($nota, $temErroAprovar, $response){
        try {
            //aprovar retorna vazio se a nota foi aprovada corretamente ou retorna com erro durante a aprovação.
            $erroNota = $this->aprovacaoLancamentosService->aprovar($nota->getNrorg(), $nota->getCdfilial(), $nota->getNrlanctonf(), $nota->getIdentrsaidop(), $response);
            if(!$erroNota) {
                $this->sucessoAprovacao[] = array("CDFILIAL"        =>  $nota->getCdfilial(),
                                                  "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                                  "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                                  "NRORG"           =>  $nota->getNrorg());
            } else {
                $temErroAprovar = true;
                
                $this->errosAprovacao[] = array("CDFILIAL"        =>  $nota->getCdfilial(),
                                                "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                                "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                                "NRORG"           =>  $nota->getNrorg(),
                                                "TPERRO"          =>  "A",
                                                "DSERRO"          =>  $erroNota);
            }
            return $temErroAprovar;
        } catch (\Exception $e) {
            $temErroAprovar = true;
            
            $this->errosAprovacao[] = array("CDFILIAL"        =>  $nota->getCdfilial(),
                                            "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                            "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                            "NRORG"           =>  $nota->getNrorg(),
                                            "TPERRO"          =>  "A",
                                            "DSERRO"          =>  $e->getMessage());
            return $temErroAprovar;
        }
    }

    public function visualizarDanfe($nota, $config){
        return $this->arrayDanfes($nota, $config, self::VISUALIZAR);
    }

    public function downloadDanfe($notasMarcadas, $config){
        $filesUrl = $this->arrayDanfes($notasMarcadas, $config, self::DOWNLOAD);
        $zipname = sys_get_temp_dir().DIRECTORY_SEPARATOR.'Danfe.zip';
        $zip = new \ZipArchive;

        if(is_file($zipname))
            $zip->open($zipname, \ZipArchive::OVERWRITE);
        else
            $zip->open($zipname, \ZipArchive::CREATE);
            
        foreach ($filesUrl as $file) {
            $filename = explode(DIRECTORY_SEPARATOR, $file);
            $zip->addFile($file, end($filename));
        }
        $caminhoZip = $zip->filename;
        $zip->close();
        
        return $zipname;
    }

    public function downloadXML($nota, $config){
        if($nota["IDSTATUSNFE"] !== 'C') $nomeArquivo = $nota["NRACESSONFE"].'-nfe.xml';
        else                             $nomeArquivo = $nota["NRACESSONFE"].'-can.xml';
        
        if(is_file(sys_get_temp_dir().'/'.$nomeArquivo)) unlink(sys_get_temp_dir().'/'.$nomeArquivo);
        
        try{
            if($nota["IDSTATUSNFE"] == 'A' || $nota["IDTIPOEMISNFE"] == 4) { //Libera xml para nfe cancelada, aceita ou EPEC.
                $xml = $this->getXML($nota, 'A');
                $dir_file = sys_get_temp_dir().'/'.$nomeArquivo;
            }
            else if($nota["IDSTATUSNFE"] == 'C') {
                $xml = $this->getXMLCancelado($nota);
                $dir_file = sys_get_temp_dir().'/'.$nomeArquivo;
            }
            file_put_contents($dir_file, $xml);
            
        }catch(\Exception $e) {
            throw new \Exception('Ocorreu erro ao tentar gerar o XML.');
        }
        
        return $dir_file;
    }

    private function arrayDanfes($notasMarcadas, $config, $modo){
        $config = json_encode($config);
        $config = $this->recolocarArrobaEmail($config);
        $config = json_decode($config);

        // $this->inicializaVarDiretorios();

        $filesUrl = array();
        foreach($notasMarcadas as $nota){
            $filesUrl[] = $this->gerarDanfe($nota, $modo, $config);
        }

        return $filesUrl;
    }
    
    private function gerarDanfe($nota, $modo, $config = array()){
        $nrlanctonf = $nota["NRLANCTONF"];
        $cdfilial = $nota["CDFILIAL"];
        $chave = $nota["NRACESSONFE"];

        $pdfDanfe = $this->pdfDir . DIRECTORY_SEPARATOR . "{$chave}-nfe.pdf";

        
        if(is_file($pdfDanfe)) unlink($pdfDanfe);
        
        $logo = $this->transmissaoNfeFactory->getDiretorioLogo($nrlanctonf, $cdfilial)[0]["DSDIRLOGOMARCA"];
        
        $xmlProt = $this->getXML($nota, self::TIPO_ARQ_XML_AUTORIZACAO, $config);
        
        $this->salvaDanfe($xmlProt, $pdfDanfe, (strip_tags($logo)), $nota);

        if($modo === self::DOWNLOAD)
            return $pdfDanfe;
        else
            return DIRECTORY_SEPARATOR . substr($pdfDanfe, strpos($pdfDanfe, "workfolder"));
      
    }

    private function getXML($nota, $tipo, $config = array()){
        if($tipo == 'A') {
            $xml = $this->transmissaoNfeFactory->getXMLEnvio($nota["CDFILIAL"], $nota["NRLANCTONF"], $tipo);
            $xmlEnvio = $this->transmissaoNfeFactory->getXMLEnvio($nota["CDFILIAL"], $nota["NRLANCTONF"], 'D')[0];
           
           if(!empty($xml) && $xml[0]["DSARQXMLNFE_CLOB"] == '' && $xml[0]["DSARQXMLNFE"] != '') {
               
                if(is_null($xmlEnvio["DSARQXMLNFE"])){
                        $xmlEnvio = $xmlEnvio["DSARQXMLNFE_CLOB"];
                }else{
                        $xmlEnvio = $xmlEnvio["DSARQXMLNFE"];
                } 
                                        
                if(!$this->nfeTools) {
                    $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
                }
                $xml = $this->addProtocolo($xmlEnvio, $xml[0]["DSARQXMLNFE"]);
            }
            else if(!empty($xmlEnvio) && isset($nota['IDTIPOEMISNFE']) && in_array($nota['IDTIPOEMISNFE'], [4, 5])) { //Libera danfe para nfe tipo EPEC ou formulário FSDA.
                $xml = $xmlEnvio["DSARQXMLNFE_CLOB"];
            }
            else {
                $xml = $xml[0]["DSARQXMLNFE_CLOB"];
            }
            return $xml;
        }
        else return $this->transmissaoNfeFactory->getXMLEnvio($nota["CDFILIAL"], $nota["NRLANCTONF"], $tipo)[0]["DSARQXMLNFE_CLOB"];
    }
    
    private function getXMLCancelado($nota) {
        $xml = $this->transmissaoNfeFactory->getXMLEnvio($nota["CDFILIAL"], $nota["NRLANCTONF"], 'C');
        if(isset($xml[0]["DSARQXMLNFE_CLOB"])) {
            return $xml[0]["DSARQXMLNFE_CLOB"];
        }
        else {
            return $xml[0]["DSARQXMLNFE"];
        }
    }

    private function salvaDanfe($xmlProt, $pdfDanfe, $logo, $nota) {
        try {
            $dirLogo = '';
            if($logo) {
                $extensaoLogo = explode(".", substr($logo, -8));
                
                if(end($extensaoLogo) !== "jpg" && end($extensaoLogo) !== "bmp") {
                    $logo = '';
                }
            }
            
            if($logo) {
                $logo = file_get_contents( self::FILESERVER_LINK.$logo);
                $dirLogo = sys_get_temp_dir().'/'.md5(uniqid()).'.jpeg';
                
                 if(is_file($dirLogo)) {
                    unlink($dirLogo);
                } 
            
                file_put_contents($dirLogo, $logo);
            } else {
                $logo = '';
            }
            
            $danfe = new Danfe($xmlProt, 'P', 'A4',  $logo, 'I', '');
            
            if(isset($nota['IDTIPOEMISNFE']) && $nota['IDTIPOEMISNFE'] == 4 && isset($nota['DTHRREGCONTNFE']) && isset($nota['NRREGDPECNFE'])) { //Libera danfe para nfe tipo EPEC.
                $danfe->monta($dirLogo, '', 'A4', 'C', $nota['NRREGDPECNFE'].' '.$nota['DTHRREGCONTNFE']);
            }
            else $danfe->monta($dirLogo);
            
            $pdf = $danfe->render($dirLogo);
            
            file_put_contents($pdfDanfe, $pdf);
            
            if($dirLogo) unlink($dirLogo);

            return $danfe;
            
            //Método montaDANFE não é mais chamado pois causa problemas com a nova versão da sped-da
            //$id = $danfe->montaDANFE();
            //$salva = $danfe->printDocument($pdfDanfe, 'F'); //Salva o PDF na pasta
        } catch(\Exception $e) {
            throw new \Exception("Ocorreu erro ao tentar gerar o Danfe.");
        }
    }

    private function atualizaStatusEmailNF($nota, $tipo){
        $notafiscal = self::getNotaFiscal($nota['NRACESSONFE']);
        if($tipo === 'T' || $tipo === 'CT'){
            $notafiscal->setIdemailnfetransp('E');
        }
        else{
            $notafiscal->setIdemailnfe('E');
        }
        $notafiscal->persist();
        $this->entityManager->flush();
    }
    
    private function atualizaStatusEmailCC($nota, $email){
        $cartaCorrecao = Cartacornfe::findByNrorgAndCdfilialAndNrlanctonf($nota["NRORG"], $nota["CDFILIAL"], $nota["NRLANCTONF"])[0];
        
        $cartaCorrecao->setIdemailcce($email['emailcce']);
        $cartaCorrecao->setIdemailccetrans($email['emailccetrans']);
        $cartaCorrecao->persist();
        $this->entityManager->flush();
    }

    public function enviarEmailNF($nota, $config, $tipo = ''){
        try {
            $config = $this->recolocarArrobaEmail(json_encode($config));
            $config = json_decode($config);
            
            $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
            // $this->inicializaVarDiretorios();
            if($this->enviaEmail($nota, $tipo)){
                $this->atualizaStatusEmailNF($nota, $tipo);
                return array('SUCCESS' => 'E-mail enviado com sucesso!!!');
            }
        } catch (\Exception $e) {
            return array('ERRORS' => $e->getMessage());
        }
    }

    public function enviarEmailCC($nota, $tipo = 'CCe', $nrseqcce, $config){
        try {
            $config = $this->recolocarArrobaEmail(json_encode($config));
            $config = json_decode($config);
            
            $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
            $email = $this->enviaEmail($nota, $tipo, $nrseqcce);
            if(!empty($email)){
                $this->atualizaStatusEmailCC($nota, $email);
                return array('SUCCESS' => 'E-mail enviado com sucesso!!!');
            }
        } catch (\Exception $e) {
            return array('ERRORS' => $e->getMessage());
        }
    }

    public function enviarEmailPendenteNFsAceitas($notasfiscais, $config){
        try {
            $config = $this->recolocarArrobaEmail($config);
            $config = json_decode($config);
            
            $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
            // $this->inicializaVarDiretorios();

            foreach($notasfiscais as $nota){
                $this->enviaEmail($nota);
            }

            return array('SUCCESS' => 'E-mail enviado com sucesso!!!');
        } catch (\Exception $e) {
            return array('ERRORS' => $e->getMessage());
        }
    }

    public function enviaEmail($nota, $tipo = '', $nrseqcce = ''){
        
        $chave = $nota['NRACESSONFE'];
        
        $dtEmi = DateUtil::getDataDeString(substr($nota['DTEMISSAO'], 0, 10))->format("Ym");
        $pathPdf = $this->pdfDir . DIRECTORY_SEPARATOR . "{$chave}-danfe.pdf";
        if(!is_file($pathPdf)){
            $pathPdf = '';
        }

        $aMails = $this->getEmailDestinatario($nota, $tipo); //se for um array vazio a classe Mail irá pegar os emails do xml
        //@todo remover linha abaixo
        $templateFile = ''; //se vazio usará o template padrão da mensagem
        
        if($tipo == 'C' || $tipo == 'CT'){
            $XML = $this->transmissaoNfeFactory->getXMLEnvio($nota["CDFILIAL"], $nota["NRLANCTONF"], self::TIPO_ARQ_XML_CANCELAMENTO);
            $XML = ($XML[0]["DSARQXMLNFE"]) ? $XML[0]["DSARQXMLNFE"] : $XML[0]["DSARQXMLNFE_CLOB"];
            $this->xml = $XML;
            return $this->enviaEmailCancelamento($nota, $XML, $aMails);
        }
        elseif ($tipo === 'CCe') {
            $XML = $this->transmissaoNfeFactory->getXMLEnvio($nota["CDFILIAL"], $nota["NRLANCTONF"], self::TIPO_ARQ_XML_CARTA_CORRECAO);
            $XML = $XML[0]["DSARQXMLNFE_CLOB"];
            $XMLAuto = $this->getXML($nota, self::TIPO_ARQ_XML_AUTORIZACAO);
            return $this->enviaEmailCartaCorrecao($nota, $XML, $XMLAuto, $aMails);
        }
        else{
            if(empty($aMails)) {
                throw new \RuntimeException("Nenhum e-mail informado.");
            }
            
            $this->gerarDanfe($nota, '');
            
            $XML = $this->getXML($nota, self::TIPO_ARQ_XML_AUTORIZACAO);
            
            $pathPdf = $this->pdfDir . DIRECTORY_SEPARATOR . "{$chave}-nfe.pdf";
            
            $this->loadDocuments($XML, $pathPdf, $this->nfeTools->config->aMailConf);
            
            $template = $this->templates["NFe"];
            
            $mensagemEmpresa = $this->transmissaoNfeFactory->getMensagemEmpresa($nota["CDEMPRESA"]);
            if(isset($mensagemEmpresa[0])) {
                if($mensagemEmpresa[0]['DSMENSEMAILNFE'] != null) {
                    $template .= $mensagemEmpresa[0]['DSMENSEMAILNFE'];
                }
                else {
                    $template .= '<p>Segue em anexo o arquivo XML da Nota Fiscal Eletrônica.</p>Atenciosamente,<br> {emitente}';
                }
            }
            else $template .= '<p>Segue em anexo o arquivo XML da Nota Fiscal Eletrônica.</p>Atenciosamente,<br> {emitente}';
            
            $template = $this->renderTemplate(
                $template,
                $this->fields->destinatario,
                $this->fields->data,
                $this->fields->numero,
                $this->fields->valor,
                $this->fields->chave,
                $this->fields->correcao,
                $this->fields->conduso
            );
            
            $this->smtpService = new SMTP(array(
                                            'host'          => $this->nfeTools->config->aMailConf->host,
                                            'username'      => $this->nfeTools->config->aMailConf->username,
                                            'password'      => $this->nfeTools->config->aMailConf->password,
                                            'port'          => $this->nfeTools->config->aMailConf->port,
                                            'smtpSecure'    => $this->nfeTools->config->aMailConf->secure
                                            ));
            
            $this->smtpService->setFrom($this->nfeTools->config->aMailConf->from);
            $this->smtpService->setFromName($this->nfeTools->config->aMailConf->fantasy);
            
            $pathXml = "/tmp/".$nota["NRACESSONFE"].'-nfe.xml';
            file_put_contents($pathXml, $this->xml);
            
            $this->smtpService->addAttachment($pathXml);
            $this->smtpService->addAttachment($pathPdf);
            
            $this->smtpService->setSubject("Emissão de Nota Fiscal Eletrônica. Chave: ".$this->fields->chave);

            $enviado = $this->smtpService->sendMail($template, $aMails);
            
            unlink($pathXml);
            unlink($pathPdf);
            
            return $enviado;
        }
    }

    private function enviaEmailCancelamento($nota, $XML, $aMails){
        if(empty($aMails)) {
            // Caso não seja informado nenhum e-mail e não tenha sido recuperado qualquer e-mail do xml
            throw new \RuntimeException('Nenhum e-mail informado.');
        }
        
        $template = $this->templates["C"];
            
        $mensagemEmpresa = $this->transmissaoNfeFactory->getMensagemEmpresa($nota["CDEMPRESA"]);
        if(isset($mensagemEmpresa[0])) {
            if($mensagemEmpresa[0]['DSMENSEMAILCANC'] != null) {
                $template .= $mensagemEmpresa[0]['DSMENSEMAILCANC'];
            }
            else {
                $template .= '<p>Segue em anexo o arquivo XML da confirmação do cancelamento da Nota Fiscal Eletrônica.</p>Atenciosamente,<br> {emitente}';
            }
        }
        else $template .= '<p>Segue em anexo o arquivo XML da confirmação do cancelamento da Nota Fiscal Eletrônica.</p>Atenciosamente,<br> {emitente}';
        
        $cliente = $this->transmissaoNfeFactory->getClienteNota($nota["NRACESSONFE"]);
        if(isset($cliente[0])) {
            $cliente = $cliente[0]["NMRAZSOCCLIE"];
        }
        else {
            $cliente = '';
        }
        
        $this->fields->destinatario = $cliente;
        $this->fields->chave  = $nota["NRACESSONFE"];
        $this->fields->numero = $nota["NRNOTAFISC"];
        
        $template = $this->renderTemplate(
            $template,
            $this->fields->destinatario,
            '',
            $this->fields->numero,
            0.0,
            $this->fields->chave
        );
        
        $this->smtpService = new SMTP(array(
                                        'host'          => $this->nfeTools->config->aMailConf->host,
                                        'username'      => $this->nfeTools->config->aMailConf->username,
                                        'password'      => $this->nfeTools->config->aMailConf->password,
                                        'port'          => $this->nfeTools->config->aMailConf->port,
                                        'smtpSecure'    => $this->nfeTools->config->aMailConf->secure
                                        ));
        
        $this->smtpService->setFrom($this->nfeTools->config->aMailConf->from);
        $this->smtpService->setFromName($this->nfeTools->config->aMailConf->fantasy);
        
        $pathXml = "/tmp/".$nota["NRACESSONFE"].'-nfe.xml';
        file_put_contents($pathXml, $this->xml);
            
        $this->smtpService->addAttachment($pathXml);
        
        $this->smtpService->setSubject("Cancelamento de Nota Fiscal Eletrônica, Chave: ".$this->fields->chave);
        
        $enviado = $this->smtpService->sendMail($template, $aMails);
        
        unlink($pathXml);

        return $enviado;
    }

    private function enviaEmailCartaCorrecao($nota, $XML, $XMLAuto, $aMails){

        if(empty($aMails)) {
            // Caso não seja informado nenhum e-mail e não tenha sido recuperado qualquer e-mail do xml
            throw new \RuntimeException("Nenhum e-mail informado.");
        }
            
        $this->loadDocuments($XML, '', $this->nfeTools->config->aMailConf);
        
        $template = $this->templates["CCe"];
            
        $mensagemEmpresa = $this->transmissaoNfeFactory->getMensagemEmpresa($nota["CDEMPRESA"]);
        if(isset($mensagemEmpresa[0])) {
            if($mensagemEmpresa[0]['DSMENSEMAILCCE'] != null) {
                $template .= $mensagemEmpresa[0]['DSMENSEMAILCCE'];
            }
            else {
                $template .= '<p>Segue em anexo o arquivo XML da Carta de Correção Eletrônica.</p>Atenciosamente,<br> {emitente}';
            }
        }
        else $template .= '<p>Segue em anexo o arquivo XML da Carta de Correção Eletrônica.</p>Atenciosamente,<br> {emitente}';
        
        $dom = new \DOMDocument('1.0', 'UTF-8');
        $dom->preserveWhiteSpace = false;
        $dom->loadXML($XMLAuto);
        $dest = $dom->getElementsByTagName('dest')->item(0);
        
        if (!empty($dest)) {
            $this->fields->destinatario = $dest->getElementsByTagName('xNome')->item(0)->nodeValue;
        }
        
        $ide = $dom->getElementsByTagName('ide')->item(0);
        $this->fields->numero = str_pad($ide->getElementsByTagName('nNF')->item(0)->nodeValue, 9, '0', STR_PAD_LEFT);
        
        $template = $this->renderTemplate(
            $template,
            $this->fields->destinatario,
            $this->fields->data,
            $this->fields->numero,
            $this->fields->valor,
            $this->fields->chave,
            $this->fields->correcao,
            $this->fields->conduso
        );
        
        $this->smtpService = new SMTP(array(
                                        'host'          => $this->nfeTools->config->aMailConf->host,
                                        'username'      => $this->nfeTools->config->aMailConf->username,
                                        'password'      => $this->nfeTools->config->aMailConf->password,
                                        'port'          => $this->nfeTools->config->aMailConf->port,
                                        'smtpSecure'    => $this->nfeTools->config->aMailConf->secure
                                        ));
        
        $this->smtpService->setFrom($this->nfeTools->config->aMailConf->from);
        $this->smtpService->setFromName($this->nfeTools->config->aMailConf->fantasy);
        
        $pathXml = "/tmp/".$nota["NRACESSONFE"].'-nfe.xml';
        file_put_contents($pathXml, $this->xml);
            
        $this->smtpService->addAttachment($pathXml);
        
        $this->smtpService->setSubject("Evento para a Nota Fiscal Eletrônica, Chave: ".$this->fields->chave);
        
        $enviado = $this->smtpService->sendMail($template, $aMails);
        
        unlink($pathXml);

        return count($aMails) == 2 ? array('emailcce' => 'E', 'emailccetrans' => 'E') : array('emailcce' => 'E', 'emailccetrans' => 'N');
    }

    protected function montaMensagemEmailCancelamento($nota)
    {
        $numero = $nota['NRNOTAFISC'];
        $chaveDeAcesso = $nota['NRACESSONFE'];
        $emitente = $nota['NMEMPRESA'];
        $destinatario = $nota['INSCRICAO'];
        $this->msgHtml = $this->renderTemplate($numero, $chaveDeAcesso, $emitente, $destinatario, 'C');
        $this->msgTxt = utf8_encode(Html2Text::convert(utf8_decode($this->msgHtml)));
        return "Cancelamento de Nota Fiscal Eletrônica. Chave: {$chaveDeAcesso}";
    }

    /**
     * Seo template estiver vazio cria o basico
     */
    protected function templateEmailCancelamento()
    {
        if (empty($this->template)) {
            $this->template = "<p>Prezados,</p>".
            "<p>E-mail referente ao cancelamento da Nota Fiscal Eletrônica nº {numero}.</p>".
            "<p>Chave de Acesso: {chavedeacesso}</p>".
            "<p>Emitente: {emitente}.
            <br>Destinatário: {destinatario}</p>".
            "<p>Segue em anexo o arquivo XML da confirmação do cancelamento da Nota Fiscal Eletrônica.</p>".
            "<p>Atenciosamente.</p>".
            "<p>{emitente}</p>";
        }
    }

    /**
     * Seo template estiver vazio cria o basico
     */
    protected function templateEmailCartaCorrecao()
    {
        if (empty($this->template)) {
            $this->template = "<p>Prezados,</p>".
            "<p>E-mail referente a Carta de Correção para a Nota Fiscal Eletrônica: {numero}</p>".
            "<p>Chave de Acesso: {chavedeacesso}</p>".
            "<p>Emitente: {emitente}.
            <br>Destinatário: {destinatario}</p>".
            "<p>Segue em anexo o arquivo XML da Carta de Correção Eletrônica</p>".
            "<p>Atenciosamente.</p>".
            "<p>{emitente}</p>";
        }
    }

    private function getEmailDestinatario($nota, $tipo)
    {
        $emailsNotasEnvio = [];
        $emailsAdic = [];
        $dadosEmail = $this->transmissaoNfeFactory->getDadosEmailNfe($nota['CDFILIAL'], $nota['NRLANCTONF']);
        $notaCompleta = self::getNotaFiscal($nota['NRACESSONFE']);

        if(!empty($dadosEmail)) {
            if($tipo === 'T' || $tipo === 'CT') {
                $emailsNotasEnvio = array($dadosEmail[0]['DSEMAILTRANSPNFE']);
            }
            else if ($tipo === 'CCe') {
                if($nota['NRINSJURTRAN']) {
                    $emailsNotasEnvio = array($dadosEmail[0]['DSEMAILDEST'], $dadosEmail[0]['DSEMAILTRANSPNFE']);
                } else {
                    $emailsNotasEnvio = array($dadosEmail[0]['DSEMAILDEST']);
                }
            }
            else {
                $emailsNotasEnvio = array($dadosEmail[0]['DSEMAILDEST']);
            }
        }
        
        /* Verificação emails adicionais */
        
        $origemNota = $notaCompleta->getIdorigdestnf();
        $cdFiliEmitRece = $notaCompleta->getIdentrsaidop() === 'E' ? $notaCompleta->getCdfiliemit() : $notaCompleta->getCdfilirece();
        
        if($tipo !== 'T' && $tipo !== 'CT') {
            if     ($origemNota === 'C') $emailsAdic = $this->transmissaoNfeFactory->getEmailsAdic('C', $notaCompleta->getCdcliente());
            else if($origemNota === 'F') $emailsAdic = $this->transmissaoNfeFactory->getEmailsAdic('F', $notaCompleta->getCdforneced());
            else if($origemNota === 'L') $emailsAdic = $this->transmissaoNfeFactory->getEmailsAdic('L', $cdFiliEmitRece);
            
            if($tipo === 'CCe') {
                $emailsAdic = array_merge($emailsAdic, $this->transmissaoNfeFactory->getEmailsAdic('T', $notaCompleta->getNrinsjurtran(), $notaCompleta->getIdtpijurtran()));
            }
        } else if($tipo === 'T' || $tipo === 'CT') {
            $emailsAdic = $this->transmissaoNfeFactory->getEmailsAdic('T', $notaCompleta->getNrinsjurtran(), $notaCompleta->getIdtpijurtran());
        }
        
        $emailsAdic = array_column($emailsAdic, 'DSEMAIL');
        $emailsNotasEnvio = array_unique(array_filter(array_merge($emailsNotasEnvio, $emailsAdic)));
        sort($emailsNotasEnvio, SORT_STRING | SORT_FLAG_CASE);
        
        return ($emailsNotasEnvio) ? : null;
    }

    /**
     * Busca Endereço de Email cadastrado na Filial
     */
    private function getEmailRemetente($filial, $organizacao)
    {
        /** @var $endefili Endefili */
        $endefili = Endefili::findOneByCdfilialAndIdtpendefiliAndNrorg(
            $filial,
            self::TIPO_END_PRINCIPAL,
            $organizacao
        );
        return $endefili->getDsemailemxmlnfe() == true ? $endefili->getDsemailemxmlnfe() : $endefili->getDsemailfili();
    }

    public function statusServico($notasfiscais, $config){
        try {
            $config = $this->recolocarArrobaEmail(json_encode($config));
            $config = json_decode($config);
            $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
            
            $tpAmb = $config->tpAmb;
            $uf = $config->siglaUF;
            $cUF = UFList::getCodeByUF($uf);
            $dataAtual = $this->transmissaoNfeFactory->getDataHoraCerta();
            $statServ = $this->transmissaoNfeFactory->getStatServ($cUF, $tpAmb);
            $resp = null; $respContingency = null; $transmite = false;
            
            if(!$statServ) { //Sempre consulta o StatusServico caso não encontre a última consulta feita.
                $resp = $this->sefazStatus($uf, $cUF, $tpAmb, $dataAtual['DATAHORACERTA'], false); 
                $respContingency = $this->sefazStatus($uf, $cUF, $tpAmb, $dataAtual['DATAHORACERTA'], true);
            }
            else {
                $cStatEsperado = [107, 108, 109];
                $dataAtualObj = DateUtil::getDataDeString($dataAtual['DATAHORACERTA'], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $statServDateObj = DateUtil::getDataDeString($statServ['DTCONSSTATNORM'] ?: DateUtil::getDataAtual(true), DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                
                //Consulta o StatusServico se a última consulta feita for maior que 3 minutos da hora atual
                //ou se o cStat gravado no banco não bater com uma resposta válida da sefaz. Por ex: NULL significa que ainda não foi consultado.
                if(DateUtil::comparaDatas(DateUtil::adicionaIntervalo($statServDateObj, 3, DateUtil::MINUTOS), $dataAtualObj)
                    || !in_array($statServ['CDSTATUS'], $cStatEsperado)) {
                        
                        $resp = $this->sefazStatus($uf, $cUF, $tpAmb, $dataAtual['DATAHORACERTA'], false);
                }
                else $this->aResposta['cStat'] = $statServ['CDSTATUS']; //Ou considera o status da última consulta feita até 3 minutos.
                
                $cStatEsperadoSVC = [107, 113, 114];
                $statServDateObjSVC = DateUtil::getDataDeString($statServ['DTRETORNOSVC'] ?: DateUtil::getDataAtual(true), DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                
                //Mesma regra porém também verifica o webservice de contingência.
                if(DateUtil::comparaDatas(DateUtil::adicionaIntervalo($statServDateObjSVC, 3, DateUtil::MINUTOS), $dataAtualObj)
                    || !in_array($statServ['CDSTATUSSVC'], $cStatEsperadoSVC)) {
                        
                        $respContingency = $this->sefazStatus($uf, $cUF, $tpAmb, $dataAtual['DATAHORACERTA'], true);
                }
                else $this->aRespostaContingency['cStat'] = $statServ['CDSTATUSSVC'];
            }
            
            /* Lista cStat Webservices StatusServico
                 * Normal
                     - 107 - Serviço em Operação.
                     - 108 - Serviço Paralisado Momentaneamente (curto prazo).
                     - 109 - Serviço Paralisado sem Previsão.
                 * Contingência
                    - 107 - Serviço SVC em Operação.
                    - 113 - SVC em processo de desativação. SVC será desabilitada para a SEFAZ-XX em dd/mm/aa às hh:mm horas.
                    - 114 – SVC desabilitada pela SEFAZ Origem. */
                    
            if(isset($resp) || isset($this->aResposta['cStat'])) {
                switch ($this->aResposta['cStat']) {
                    case self::SERVICO_PARALISADO_TEMPORARIAMENTE:
                        $mensagem = 'Serviço de recepção da Secretaria Nacional da Fazenda (SEFAZ) ou do órgão estadual ligado a
                                     ela paralisado temporariamente. Caso deseje, poderá utilizar algum dos modelos operacionais
                                     disponíveis para emissão em contingência, ou se preferir, tente novamente mais tarde.';
                        return ['STATUS'               => $mensagem,
                                'aResposta'            => $this->aResposta,
                                'aRespostaContingency' => $this->aRespostaContingency,
                                'contingency'          => false];
                        break;
                    case self::SERVICO_PARALISADO_SEM_PREVISAO:
                        $mensagem = 'Serviço de recepção da Secretaria Nacional da Fazenda (SEFAZ) ou do órgão estadual ligado a
                                     ela paralisado sem previsão de retorno. Caso deseje, poderá utilizar algum dos modelos operacionais
                                     disponíveis para emissão em contingência, ou se preferir, tente novamente mais tarde.';
                        return ['STATUS'               => $mensagem,
                                'aResposta'            => $this->aResposta,
                                'aRespostaContingency' => $this->aRespostaContingency,
                                'contingency'          => false];
                        break;
                }
                $transmite = true;
            }
            if(isset($respContingency) || isset($this->aRespostaContingency['cStat'])) {
                if($this->aRespostaContingency['cStat'] == self::SERVICO_ATIVO) {
                    $mensagem = 'Atenção! Foi identificado que o Serviço Virtual de Contingência (SVC-AN/SVC-RS) encontra-se habilitado para
                                 a Secretaria Nacional da Fazenda (SEFAZ) ou do órgão estadual ligado a ela. Favor identificar o tipo de transmissão
                                 que deseja utilizar.';
                    return ['STATUS'               => $mensagem,
                            'aResposta'            => $this->aResposta,
                            'aRespostaContingency' => $this->aRespostaContingency,
                            'contingency'          => true];
                }
                $transmite = true;
            }
            if(!isset($resp) && !isset($this->aResposta['cStat']) && !isset($respContingency) && !isset($this->aRespostaContingency['cStat'])) {
                return ['STATUS'               => 'O seguinte erro ocorreu na tentativa de consultar o status do serviço prestado pelo Portal
                                                   da Secretaria da Fazenda Estadual: "Não houve retorno da comunicação com a SEFAZ".
                                                   Você pode optar por tentar utilizar a transmissão da NF-e normalmente ou se preferir
                                                   poderá utilizar algum dos modelos operacionais disponíveis para emissão em contingência,
                                                   ou ainda, poderá tentar novamente mais tarde.',
                        'aResposta'            => $this->aResposta,
                        'aRespostaContingency' => $this->aRespostaContingency,
                        'contingency'          => false];
            }
            
            if($transmite && $this->aResposta['cStat'] == self::SERVICO_ATIVO)
                return $this->transmitirNotas($notasfiscais, $config);
                
        } catch (\Exception $e) {
            return ['STATUS'               => "O seguinte erro ocorreu na tentativa de consultar o status do serviço prestado pelo Portal
                                               da Secretaria da Fazenda Estadual: [".$e->getMessage()."].
                                               Você pode optar por tentar utilizar a transmissão da NF-e normalmente ou se preferir
                                               poderá utilizar algum dos modelos operacionais disponíveis para emissão em contingência,
                                               ou ainda, poderá tentar novamente mais tarde.",
                    'aResposta'            => $this->aResposta,
                    'aRespostaContingency' => $this->aRespostaContingency,
                    'contingency'          => false];
        }
    }
    
    private static function getEntidadesNotasFiscais($notasFiscais)
    {
        $retorno = array();
        foreach ($notasFiscais as $nota) {
            $retorno[] = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['NRORG']);
        }
        return $retorno;
    }

    public function transmitirNotas($notasfiscais, $config, $tipo = ''){        
        try {
            $config = $this->recolocarArrobaEmail(json_encode($config));
            $config = json_decode($config);
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
            $nmOrg = $this->transmissaoNfeFactory->getOrganizacao($config->dadosFilial->NRORG)["NMORGANIZACAO"];
            
            if($nmOrg == 'GRSA - BRASIL') {
                $notas = array();
                foreach($notasfiscais as $nota) {
                    $paramnfbloqaffil = $this->transmissaoNfeFactory->verificaTipoOperac($nota["CDTIPOOPER"], $nota["CDFILIAL"], $nota["CDTIPOOPER"]);
                    if(isset($paramnfbloqaffil[0])) {
                        if($paramnfbloqaffil[0]["IDLIBERATPOPFIL"] == 'S') {
                            array_push($notas, $nota);
                        }
                    }
                    else array_push($notas, $nota);
                }
            }
            else {
                $notas = $notasfiscais;
            }
            
            if($notas) {
                $notasfiscais = $notas;
            }
            else {
                throw new \Exception("Não foi possivel realizar a transmissão. O tipo de operação das notas não é permitido para a unidade.");
            }
            
            $nfArray = $notasfiscais;
            $notasfiscais = $this->getEntidadesNotasFiscais($notasfiscais);
            $notasfiscais = $this->setDtemissao($notasfiscais);
            $array_erros = $this->validacoesTransmissaoNfeService->validaNFe($notasfiscais, $this->nfeTools->config);
            $paramgeral  = $this->transmissaoNfeFactory->getParamGeral()[0];

            if($array_erros){
                $this->nfeTools->errors = $array_erros;
                throw new \Exception('ERRORS');
            }

            if($tipo){
                $type = ($tipo == 'FSDA' || $tipo == 'EPEC') ? $tipo : null;
                
                $this->nfeTools->contingency->activate($config->siglaUF, 'DANFE em Contingência - impresso em decorrência de problemas técnicos', $type);
                $this->tpEmis = $this->nfeTools->contingency->tpEmis;
            }
            else{
                if($this->nfeTools->contingency->type) $this->nfeTools->contingency->deactivate();
                $this->tpEmis = self::TIPO_EMISSAO_NORMAL;
            }
            
            $this->idLote = $this->novoCodigo->getCodeByTableName(
                self::CONTADOR_NOVOCODIGO_NRLOTE . $this->nfeTools->config->cdempresa,
                $this->nfeTools->config->nrorg, 1, self::TAMANHO_LOTENFE);
            
            foreach($notasfiscais as $nota){
                $this->nota = $nota;
                if(in_array($nota->getIdstatusnfe(), array('P', 'R'))){
                    $this->corpoNF = $this->transmissaoNfeFactory->getCorpoNotaFiscal($nota->getCdfilial(), $nota->getNrlanctonf(), $nota->getIdorigdestnf(), $nota->getIdentrsaidop(), $nota->getNrorg());
                    $this->nfeTools->config->cnpj = $this->corpoNF["NRINSCESTEMIT"];
                    $this->geraVencimento($nota);
                    $resp = $this->xmlNFeService->geraXML($nota, $this->corpoNF, $this->nfeTools, $this->tpEmis);
                    
                    if ($resp) {
                        if(!empty($resp['erros'])) throw new \Exception('ERRORS');
                        $this->xml = $resp['xml'];
                        $this->chave = $resp['chave'];
                        $this->numeroNotaFiscal = $resp['numero'];
                        $this->gravaNumeroNotafiscalEle();
                        
                        $this->xml = !$tipo ? $this->assinaXml($this->xml) : $this->nfeTools->signNFe($this->xml);
                        // $this->salvaXmlNfe($this->entDir, $this->chave, $this->xml);
                        $this->aXml[] = $this->xml;
                        $this->nota->setNracessonfe($this->chave);
                        $this->aNota[] = $this->nota;
                        $this->gravaXmlBanco($nota->getNrorg(),
                                       $this->xml,
                                       $nota->getCdfilial(),
                                       $nota->getNrlanctonf(),
                                       self::TIPO_ARQ_XML_DADOS_NOTA_FISCAL);
                    } else {
                        throw new \Exception('ERRORS');
                    }
                }
            }
            
            if(!$tipo || $tipo == 'SVC') { //Transmissão normal ou em contingência SVC.
                $resp = $this->enviaLote();
    
                if ($resp) {
                    $this->insereLoteNfe();
                    foreach($this->aNota as $nota){
                        $this->updateNotaFiscal($nota, $tipo);
                        $this->updateNotaOl($nota);
                        if($paramgeral["IDREPLNRDT"] == 'S') {
                            $this->updateNotaReplicada($nota);
                        }
                        $this->insereRelacnflotenfe($nota);
                        $this->montaArrayRetorno($nota);
                        // $this->salvaXmlNfe($this->envDir, $nota->getNracessonfe(), '', $this->valDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
                        // $this->unlinkXMLs($nota);
                    }
                    $connection->commit();
                }
                else{
                    throw new \Exception($resp);
                }
            }
            else if($tipo == 'EPEC') { //Transmissão EPEC. Biblioteca só transmite um lote com uma nota epec por vez.
                $resp = $this->enviaLoteEPEC();
                
                if($resp) {
                    if($this->aResposta['cStat'] == self::ID_LOTE_EVENTO_PROCESSADO) {
                        if($this->aResposta['evento'][0]['cStat'] == self::EVENTO_EPEC_AUTORIZADO ||
                           $this->aResposta['evento'][0]['cStat'] == self::EVENTO_REGISTRADO_N_VINCULADO) {
                        
                            $this->updateNotaFiscal($this->aNota[0], $tipo);
                            $this->transmissaoNfeFactory->updateDataNFeEPEC($this->aResposta['evento'][0]['chNFe'],
                                                                            date_format(date_create($this->aResposta['evento'][0]['dhRegEvento']), 'd/m/Y H:i:s'),
                                                                            $this->aResposta['evento'][0]['nProt'],
                                                                            $this->aResposta['evento'][0]['xMotivo'],
                                                                            $this->aResposta['evento'][0]['cStat'],
                                                                            $this->nfeTools->config->tpAmb,
                                                                            $this->tpEmis,
                                                                            'S', 'S');
                            $this->montaArrayRetorno($this->aNota[0]);
                            $connection->commit();
                        }
                        else {
                            $this->transmissaoNfeFactory->updateDataNFeEPEC($this->aResposta['evento'][0]['chNFe'],
                                                                            null,
                                                                            null,
                                                                            $this->aResposta['evento'][0]['xMotivo'],
                                                                            $this->aResposta['evento'][0]['cStat'],
                                                                            $this->nfeTools->config->tpAmb,
                                                                            0, 'N', 'N');
                            $this->montaArrayRetorno($this->aNota[0]);
                            $connection->commit();
                        }
                    }
                    else { //Lote EPEC não processado.
                        throw new \Exception($this->aResposta['cStat'].' - '.$this->aResposta['xMotivo']);
                    }
                }
                else{
                    throw new \Exception($resp);
                }
            }
            else if($tipo == 'FSDA') { //Transmissão FSDA.
                foreach($this->aNota as $nota){
                    $this->updateNotaFiscal($nota, $tipo);
                    $this->montaArrayRetorno($nota);
                }
                $connection->commit();
            }
            return array('SUCCESS' => $this->aRetorno, 'XMLRESP' => $resp, 'RETORNO' => $this->aResposta, 'ENVIO' => $this->aXml);
            
        } catch (\Exception $e) {
            $connection->rollback();
            
            if($e->getMessage() === 'ERRORS') {
                if(!isset($this->nfeTools->errors[0])) return array('ERRORS' => $this->nfeTools->errors[0]);
                else if(isset($resp["errors"])) {
                    return array('ERRORS' => $resp["errors"], 'RETORNO' => $this->aResposta, 'ENVIO' => $this->aXml);
                }
                else if($array_erros) {
                    return array('ERRORS' => $array_erros, 'RETORNO' => $this->aResposta, 'ENVIO' => $this->aXml);
                }
                else {
                    return array('ERRORS' => $e->getMessage(), 'RETORNO' => $this->aResposta, 'ENVIO' => $this->aXml);
                }
            }
            else {
                return array('ERRORS' => $e->getMessage(), 'RETORNO' => $this->aResposta, 'ENVIO' => $this->aXml);
            }
        }
    }
    
    public function setDtemissao($notasfiscais) {

        foreach($notasfiscais as &$nota) {
            $dtEmissao    = $this->transmissaoNfeFactory->getDtEmissaoTransmite($nota->getNrlanctonf(), $nota->getCdfilial())[0]['DTEMISSAO'];
            if(substr($dtEmissao, 0, 10) == $nota->getDtentrsaid()->format('d/m/Y')) $dtEmissao = $nota->getDtentrsaid()->format('d/m/Y H:i:s');
            $dtEmissao    = str_replace('/', '-', $dtEmissao);
            if(!$dtEmissao) {
                $dtEmissao = $nota->getDtentrsaid()->format('d/m/Y H:i:s');
            }
            $dtEmissao = str_replace('/', '-', $dtEmissao);
            $dtEmissao    = new \DateTime($dtEmissao);
            $dtEmissao    = new \DateTime($dtEmissao->format("d-m-Y H:i:s"));
            $nota->setDtemissao($dtEmissao);
            $nota->persist();
            $this->entityManager->flush();
        }

        return $notasfiscais;
    }

    public function atualizaLote($lotesMarcados, $config){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $config = json_encode($config);
            $config = $this->recolocarArrobaEmail($config);
            $config = json_decode($config);
            
            $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
            // $this->inicializaVarDiretorios();
            foreach($lotesMarcados as $lote){
                $retornoSefaz = $this->consultaRecibo($lote);
                $this->protocolaNota($lote, $retornoSefaz);
            }

            $connection->commit();
            return array('SUCCESS' => $this->aResposta, 'NOTAS_SUCCESS' => $this->notasLoteSuccess);
        } catch (\Exception $e) {
            $connection->rollback();
            return array('ERRORS' => $e->getMessage());
        }
    }

    public function cancelarTransmissao($nota, $config){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $config = json_encode($config);
            $config = $this->recolocarArrobaEmail($config);
            $config = json_decode($config);
            $config->cnpj = substr($nota["NRACESSONFE"] , 6 , 14);
            $cStatPermitidos = [101, 135, 136, 151, 155];
            
            /*101 - Cancelamento de NF-e homologado
             135 - Evento registrado e vinculado a NF-e
             136 - Evento registrado, mas não vinculado a NF-e
             151 - Cancelamento de NF-e homologado fora do prazo
             155 - Cancelamento de NF-e homologado fora do prazo (cancelamento por recepção de evento)*/
            
            $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
            // $this->inicializaVarDiretorios();
            
            $xmlCancel = $this->cancelaTransmissao($nota);
            if(in_array($this->aResposta['evento'][0]['cStat'], $cStatPermitidos)) {
                $xmlCancel = $this->trataXMLRetorno($this->nfeTools->lastRequest, $this->nfeTools->lastResponse);
                $resp = $this->gerarNovaNotaAposCancelamento($nota, $xmlCancel);
                $this->gravaXmlBancoCancelado($nota['NRORG'], $xmlCancel, $nota['CDFILIAL'], $nota['NRLANCTONF'], $resp['CDFILIAL'], $resp['NRLANCTONF']);

                $connection->commit();
                return array('SUCCESS' => $resp);
            }
            else{
                throw new \Exception($this->aResposta['evento'][0]['xMotivo']);
            }
        } catch (\Exception $e) {
            $connection->rollback();
            return array('ERRORS' => $e->getMessage());
        }
    }

    public function enviarCartaCorrecao($nota, $dscorrecaonfe, $nrseqcce, $config){
        $config = json_encode($config);
        $config = $this->recolocarArrobaEmail($config);
        $config = json_decode($config);
        $config->cnpj = substr($nota["NRACESSONFE"] , 6 , 14);
        
        $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
        // $this->inicializaVarDiretorios();

        $xmlCCe = $this->enviaCartaCorrecao($nota, $dscorrecaonfe, $nrseqcce);
        
        if($this->aResposta['evento'][0]['cStat'] == '135' || $this->aResposta['evento'][0]['cStat'] == '136'){ //Evento registrado e vinculado a NF-e / Evento registrado, mas não vinculado a NF-e.
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $xmlCCe = $this->trataXMLRetorno($this->nfeTools->lastRequest, $this->nfeTools->lastResponse);
            $this->gravaXmlBanco($nota['NRORG'], $xmlCCe, $nota['CDFILIAL'], $nota['NRLANCTONF'], self::TIPO_ARQ_XML_CARTA_CORRECAO);
            $this->insereCarta($nota['NRORG'], $nota['CDFILIAL'], $nota['NRLANCTONF'], $nrseqcce, $dscorrecaonfe,
                               $this->aResposta['evento'][0]['cStat'], $this->aResposta['evento'][0]['xMotivo'], $this->aResposta['evento'][0]['dhRegEvento'],
                               $this->aResposta['evento'][0]['nProt'], $this->aResposta['idLote'], $nota['CDEMPRESA']);
            
            $connection->commit();
            
            return array('SUCCESS' => $this->aResposta['evento'][0]);
        }
        else{
            if($this->aResposta['evento'][0]['cStat'] == '573') { //Rejeição: Duplicidade de Evento.
                $connection = $this->entityManager->getConnection();
                $connection->beginTransaction();
                $this->transmissaoNfeFactory->atualizaCCeDuplicidade($nota['CDFILIAL'], $nota['NRLANCTONF']);
                $connection->commit();
            }
            return array('ERRORS' => $this->aResposta['evento'][0]['xMotivo']);
        }
    }

    private function consultaRecibo($lote){
        $this->aResposta = [];
        $recibo = $lote['NRRECIBONFE'];
        $tpEmisLote = $lote['IDTIPOEMISLOTE'];
        $tpAmb = $this->nfeTools->config->tpAmb;
        
        if($tpEmisLote == self::TIPO_EMISSAO_SVAN_AN || $tpEmisLote == self::TIPO_EMISSAO_SVAN_RS)
            $this->nfeTools->contingency->activate($this->nfeTools->config->siglaUF, 'Atualização do lote em contingência.');
        else if($this->nfeTools->contingency->type)
            $this->nfeTools->contingency->deactivate();
        
        $resp = $this->nfeTools->sefazConsultaRecibo($recibo, $tpAmb, $this->aResposta);
        
        $this->aResposta = ReturnNFe::readReturnSefaz('NfeRetAutorizacao', $resp);
        
        return $resp;
    }

    private function cancelaTransmissao($nota){
        $this->aResposta = array();
        $chave = $nota['NRACESSONFE'];
        $nProt = $nota['NRPROTOCOLONFE'];
        $tpAmb = $this->nfeTools->config->tpAmb;
        $xJust = $nota['DSMOTIVOCANCNFE'];
        
        if($nota['IDTIPOEMISNFE'] == self::TIPO_EMISSAO_SVAN_AN || $nota['IDTIPOEMISNFE'] == self::TIPO_EMISSAO_SVAN_RS)
            $this->nfeTools->contingency->activate($this->nfeTools->config->siglaUF, 'Cancelamento da nota em contingência.');
        else if($this->nfeTools->contingency->type)
            $this->nfeTools->contingency->deactivate();
        
        $resp = $this->nfeTools->sefazCancela($chave, $xJust, $nProt, $this->aResposta);
        
        $this->aResposta = ReturnNFe::readReturnSefaz('RecepcaoEvento', $resp);

        return $resp;
    }

    private function enviaCartaCorrecao($nota, $dscorrecaonfe, $nrseqcce){
        $this->aResposta = array();
        $chave = $nota['NRACESSONFE'];
        $tpAmb = $this->nfeTools->config->tpAmb;
        
        $nrseqcce = (int) $nrseqcce;
        
        $resp = $this->nfeTools->sefazCCe($chave, $dscorrecaonfe, $nrseqcce, $this->aResposta);
        
        $this->aResposta = ReturnNFe::readReturnSefaz('RecepcaoEvento', $resp);
        
        return $resp;
    }

    private function trataXMLRetorno($xmlEnvio, $xmlResposta) {
        
        $xmlEnvio = explode("<evento", $xmlEnvio);
        $xmlEnvio = '<evento '. $xmlEnvio[1];
        $xmlEnvio = explode("</evento>", $xmlEnvio);
        $xmlEnvio = $xmlEnvio[0]."</evento>";
        
        $xmlCarta = '<procEventoNFe xmlns="http://www.portalfiscal.inf.br/nfe" versao="1.00">'.$xmlEnvio;
        
        $xmlResposta = explode("<retEvento", $xmlResposta);
        
        $xmlResposta = '<retEvento'.$xmlResposta[1];
        
        $xmlResposta = explode("</retEnvEvento>", $xmlResposta);
        
        $xmlResposta = $xmlResposta[0]."</procEventoNFe>";
        
        $xmlCarta.= $xmlResposta;
        
        return $xmlCarta;
        
    }
    
    /**
     * Este Método gera uma nova Nota Fiscal com os mesmos dados da Nota Original mais os dados do Cancelamento,
     * a Nota Original será alterada para pendente.
     * @param Notafiscal $notafiscalorigem
     * @param            $xmlCancelamento string
     * @param            $isSincronizacao boolean
     * @throws \Exception
     * @internal param $justificativa
     */
    private function gerarNovaNotaAposCancelamento($nota, $xmlCancelamento, $isSincronizacao = false)
    {
        try {
            $notafiscalorigem = Notafiscal::findOneByCdfilialAndNrlanctonfAndIdentrsaidopAndNrorg(
                $nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP'], $nota['NRORG']);

            $doc                    = new \DOMDocument();
            $doc->preservWhiteSpace = false; //elimina espaços em branco
            $doc->formatOutput      = false;
            $doc->loadXML($xmlCancelamento);

            if($isSincronizacao){
                $cstat = $this->getTagXml($xmlCancelamento,'cStat');
                $justificativa = $this->getTagXml($xmlCancelamento,'xJust');
                $motivo = $this->getTagXml($xmlCancelamento,'xMotivo');
                $dthoraprot = $this->getTagXml($xmlCancelamento,'dhRegEvento');
                $this->formatDtHrProt($dthoraprot);
                $tpEvento = $this->getTagXml($xmlCancelamento,'tpEvento');
            }
            else{
                //Status
                $cstat = $doc->getElementsByTagName('cStat')->item(0)->nodeValue;
                //Justificativa
                $justificativa = $doc->getElementsByTagName('xJust')->item(0)->nodeValue;
                //Motivo
                $motivo = $doc->getElementsByTagName('xMotivo')->item(0)->nodeValue;
                //Data e Hora do Protocolo
                $dthoraprot = $doc->getElementsByTagName('dhRegEvento')->item(0)->nodeValue;
                $this->formatDtHrProt($dthoraprot);
                // Tipo do Evento
                $tpEvento = $doc->getElementsByTagName('tpEvento')->item(0)->nodeValue; 
            }
            
            if ($tpEvento == self::EVENTO_CANCELAMENTO_REG) {
                $motivo .= "(Dados da NF Cancelada: Filial: {$notafiscalorigem->getCdfilial()}, Lançamento: {$notafiscalorigem->getNrlanctonf()})";
            }
            
            //Insere a nova Nota Fiscal
            /** @var $novaNotafiscal NotaFiscal */
            $novaNotafiscal = clone $notafiscalorigem;
            $novaNotafiscal->setNrlanctonf(
                $this->novoCodigo->getCodeByTableName(
                    'LANCNF/FILIAL' . $notafiscalorigem->getCdfilial(), $notafiscalorigem->getNrorg())
            );
            $novaNotafiscal->setIdbloqueado(Notafiscal::NOTA_FISCAL_BLOQUEADA);
            $novaNotafiscal->setIdstatus(Notafiscal::CANCELADO);
            $novaNotafiscal->setDtcancelanf($dthoraprot);
            // $novaNotafiscal->setCdopecancnf(Ambiente::getCdOperador());
            $novaNotafiscal->setIdstatusnfe(Notafiscal::CANCELADO);
            $novaNotafiscal->setDsmotivocancnfe($justificativa);
            $novaNotafiscal->setDsobsstatusnfe($motivo);
            $novaNotafiscal->persist();
            $this->entityManager->flush();
            
            //Updata manualmente a Nota Fiscal de Origem em alguns campos que não existem na entidade notafiscal.
            $this->transmissaoNfeFactory->updateNFeCancelEPEC($nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP'], $nota['NRORG']);
            
            //Updata a Nota Fiscal de Origem
            $notafiscalorigem->setNrnotafisc(null);
            $notafiscalorigem->setNracessonfe(null);
            $notafiscalorigem->setNrprotocolonfe(null);
            $notafiscalorigem->setIdstatusnfe(Notafiscal::PENDENTE);
            $notafiscalorigem->setNrnotafiscalele(null);
            $notafiscalorigem->setDsobsstatusnfe(null);
            $notafiscalorigem->setCdemplotenfe(null);
            $notafiscalorigem->setNrlotenfe(null);
            $notafiscalorigem->setIdbloqueado(Notafiscal::NOTA_FISCAL_NAO_BLOQUEADA);
            $notafiscalorigem->setIdstatus(Notafiscal::PENDENTE);
            $notafiscalorigem->setDtsaidemis(null);
            $notafiscalorigem->setDtemissao(null);
            $notafiscalorigem->setCdoperapro(null);
            $notafiscalorigem->setNrlanctbdoc(null);
            $notafiscalorigem->setDtexprnf(null);
            $notafiscalorigem->setNrexprnf(null);
            $notafiscalorigem->setIdtipoemisnfe(NotaFiscal::IDTIPOEMISNFE_NAO_EMITIDO);
            $notafiscalorigem->setIdemailnfe('P');
            $notafiscalorigem->setIdemailnfetransp('P');
            $notafiscalorigem->setIdtpambnfe(Notafiscal::IDTPAMBNFE_0);
            $notafiscalorigem->setCdbarraconting(null);
            $notafiscalorigem->setDthrprotocolnfe(null);
            $notafiscalorigem->setCdparamemisre(null);
            $notafiscalorigem->setNrseqcapadoc(null);
            $notafiscalorigem->setNrregdpecnfe(null);
            $notafiscalorigem->setDthrregcontnfe(null);
            $notafiscalorigem->persist();
           
            //Atualiza carta de correção caso exista
            $cartaCorrecao = Cartacornfe::findByNrorgAndCdfilialAndNrlanctonf($notafiscalorigem->getNrorg(), $notafiscalorigem->getCdfilial(), $notafiscalorigem->getNrlanctonf());
            foreach($cartaCorrecao as $carta){
                /** @var Cartacornfe $novaCartaCorrecao */
                $novaCartaCorrecao = clone $carta;
                $novaCartaCorrecao->setNrlanctonf($novaNotafiscal->getNrlanctonf());
                $novaCartaCorrecao->persist();
                $this->entityManager->remove($carta);
            }

            $this->entityManager->flush();

            return array('DTHRPROTOCOLNFE' => $dthoraprot->format('d/m/Y H:i:s'),
                    'DSOBSSTATUSNFE' => $motivo,
                    'CDFILIAL' => $novaNotafiscal->getCdfilial(),
                    'NRLANCTONF' => $novaNotafiscal->getNrlanctonf());
                    
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function protocolaNota($lote, $retornoSefaz){

        switch ($this->aResposta['cStat']) {
            case self::ID_LOTE_PROCESSADO:
                $this->protocoloLoteProcessado($lote['NRLOTENFE'], $lote['NRRECIBONFE'], $retornoSefaz);
                break;
            case self::ID_LOTE_EM_PROCESSAMENTO:
                $this->protocoloLoteEmProcessamento($lote['NRLOTENFE']);
                break;
            case self::ID_LOTE_NAO_LOCALIZADO:
                $this->protocoloLoteNaoLocalizado($lote['NRLOTENFE']);
                break;
            default:
                $this->protocoloProblemaReciboOuCnpj($lote['NRLOTENFE']);
                break;
        }
    }
    
    public function consultaSituacaoAtualNF($chaveNfe){
        $nota = self::getNotaFiscal($chaveNfe);
        if(empty($nota)){
            return [];
        }
        $cdfilial = $nota->getCdfilial();
        $nrlanctonf = $nota->getNrlanctonf();
        
        $idendemisfil = $this->transmissaoNfeFactory->getIdendemisfil($cdfilial);
        
        $config = $this->montaConfigTransmissao($nota->getCdempresa(), $nota->getCdInscesta(), $cdfilial, $idendemisfil);
        
        if(gettype($nota) != 'object') {
            $nota = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['NRORG']);
        }
            
        $config = json_encode($config);
        $config = $this->recolocarArrobaEmail($config);
        $config = json_decode($config);
        $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
        $retornoSefaz = $this->nfeTools->sefazConsultaChave($nota->getNracessonfe(), $config->tpAmb, $this->aResposta);
        
        $aResposta = ReturnNFe::readReturnSefaz('NfeConsultaProtocolo', $retornoSefaz);
        
       return $aResposta;
        
    }
    
    public function sincronizaDadosNota($chave, $isSincronizaLote) {
        $nota = self::getNotaFiscal($chave);
        if(empty($nota)){
            return array('ERRO' => 'Não foi possivel sincronizar. Nota inexistente no sistema');
        }
        $situacao = $this->transmissaoNfeFactory->getSituacaoNota($chave);
        $status = $situacao[0]["IDSTATUS"];
        $statusnfe = $situacao[0]["IDSTATUSNFE"];
        $nrorg = $situacao[0]["NRORG"];
        $cdempresa = $situacao[0]["CDEMPRESA"];
        $atualizouCarta = false;
        
        
        $infoSefaz = $this->consultaSituacaoAtualNF($chave);
        
        if(empty($infoSefaz)){
            return array('ERRO' => 'Não foi possivel sincronizar. Tente novamente mais tarde.');
        }
        
        if($infoSefaz["cStat"] == 101){
            $situacaoSefaz = "C";
        }else if($infoSefaz["cStat"] == 110 || $infoSefaz["cStat"] == 301 || $infoSefaz["cStat"] == 302){
             $situacaoSefaz = "D";
        }else if($infoSefaz["cStat"] == 100){
             $situacaoSefaz = "A";
        }else{
             $situacaoSefaz = "R";
        }
        
        if($situacaoSefaz != "R"){
            if($status == "A"){
                //atualiza apenas informações de carta de correção
                foreach ($infoSefaz['aEvent'] as $eventos) {
                    foreach ($eventos as $evento) {
                        if($evento["tpEvento"] == 110110){
                            $cdstatus = $evento["cStat"];
                            $dsstatus = $evento["xMotivo"];
                            $nrseqEvento = str_pad($evento["nSeqEvento"], 2, "0", STR_PAD_LEFT);
                            //$nrseqEvento = $evento["nSeqEvento"];
                            //$dataFinal = $this->constroiData(substr($evento["dhRegEvento"], 0, 10));
                            $dataFinal = $evento["dhRegEvento"];
                            $protocolo = $evento["nProt"];
                            $dscorrecaonfe = $evento['xCorrecao'];
                            
                            $carta = $this->transmissaoNfeFactory->verificaCartaCor($nota->getCdfilial(),$nota->getNrlanctonf(), $nrseqEvento);
                            if(!$carta){
                                $atualizouCarta = true;
                              $this->insereCarta($nota->getNrorg(),$nota->getCdfilial(),$nota->getNrlanctonf(), $nrseqEvento,$dscorrecaonfe,
                              $cdstatus,$dsstatus,$dataFinal, $protocolo, " ", $cdempresa);
                              
                            } 
                            
                            $xmlCCe = $this->trataXML($this->nfeTools->lastRequest, $this->nfeTools->lastResponse,$evento["nSeqEvento"]);
                            $this->gravaXmlBanco($nota->getNrorg(), $xmlCCe, $nota->getCdfilial(),$nota->getNrlanctonf(), self::TIPO_ARQ_XML_CARTA_CORRECAO); 
                        }
                    }  
                }
                if($atualizouCarta){
                   return array('SUCESS' => 'Cartas de correção sincronizadas com sucesso.', 'APROVADA' => 'TRUE'); 
                }else{
                   return array('SUCESS' => 'Não existem informações para serem sincronizadas', 'APROVADA' => 'TRUE');  
                }
               
            }else{
                if($situacaoSefaz == "C"){
                    foreach ($infoSefaz['aEvent'] as $eventos) {
                        foreach ($eventos as $evento) {
                            if($evento["tpEvento"] == 110110){
                                $cdstatus = $evento["cStat"];
                                $dsstatus = $evento["xMotivo"];
                                $nrseqEvento = str_pad($evento["nSeqEvento"], 2, "0", STR_PAD_LEFT);
                                //$nrseqEvento = $evento["nSeqEvento"];
                                //$dataFinal = $this->constroiData(substr($evento["dhRegEvento"], 0, 10));
                                $dataFinal = $evento["dhRegEvento"];
                                $protocolo = $evento["nProt"];
                                $dscorrecaonfe = $evento['xCorrecao'];
                                
                                $carta = $this->transmissaoNfeFactory->verificaCartaCor($nota->getCdfilial(),$nota->getNrlanctonf(), $nrseqEvento);
                                if(!$carta){
                                  $this->insereCarta($nota->getNrorg(),$nota->getCdfilial(),$nota->getNrlanctonf(), $nrseqEvento,$dscorrecaonfe,
                                  $cdstatus,$dsstatus,$dataFinal, $protocolo, " ", $cdempresa);
                                  
                                } 
                                $xmlCCe = $this->trataXML($this->nfeTools->lastRequest, $this->nfeTools->lastResponse,$evento["nSeqEvento"]);
                                $this->gravaXmlBanco($nota->getNrorg(), $xmlCCe, $nota->getCdfilial(),$nota->getNrlanctonf(), self::TIPO_ARQ_XML_CARTA_CORRECAO); 
                            }
                        }  
                    }
                    if($situacaoSefaz != $statusnfe){
                        //cancela nota
                        $novaNota = array();
                        $novaNota['CDFILIAL'] = $nota->getCdfilial();
                        $novaNota['NRLANCTONF'] = $nota->getNrlanctonf(); 
                        $novaNota['IDENTRSAIDOP'] = $nota->getIdentrsaidop(); 
                        $novaNota['NRORG'] = $nota->getNrorg();
                        $xmlCCe = $this->trataXML($this->nfeTools->lastRequest, $this->nfeTools->lastResponse,$evento["nSeqEvento"]);
                        $resp = $this->gerarNovaNotaAposCancelamento($novaNota, $xmlCCe, true);
                        $this->transmissaoNfeFactory->insereXmlCancelado($novaNota['CDFILIAL'], $novaNota['NRLANCTONF'], $resp['CDFILIAL'], $resp['NRLANCTONF']);
                        $this->transmissaoNfeFactory->removeXmlCancelado($novaNota['CDFILIAL'], $novaNota['NRLANCTONF']);
                        $xml = $this->transmissaoNfeFactory->verificaXml($nota->getCdfilial(),$nota->getNrlanctonf(), self::TIPO_ARQ_XML_CANCELAMENTO);
                        if(!$xml){
                           $this->gravaXmlBanco($novaNota['NRORG'], $xmlCCe, $resp['CDFILIAL'], $resp['NRLANCTONF'], self::TIPO_ARQ_XML_CANCELAMENTO); 
                        }
                    }
                }else if($situacaoSefaz == "D"){
                    foreach ($infoSefaz['aEvent'] as $eventos) {
                        foreach ($eventos as $evento) {
                            if($evento["tpEvento"] == 110110){
                                $cdstatus = $evento["cStat"];
                                $dsstatus = $evento["xMotivo"];
                                $nrseqEvento = str_pad($evento["nSeqEvento"], 2, "0", STR_PAD_LEFT);
                                //$nrseqEvento = $evento["nSeqEvento"];
                                //$dataFinal = $this->constroiData(substr($evento["dhRegEvento"], 0, 10));
                                $dataFinal = $evento["dhRegEvento"];
                                $protocolo = $evento["nProt"];
                                $dscorrecaonfe = $evento['xCorrecao'];
                                
                                $carta = $this->transmissaoNfeFactory->verificaCartaCor($nota->getCdfilial(),$nota->getNrlanctonf(), $nrseqEvento);
                                if(!$carta){
                                  $this->insereCarta($nota->getNrorg(),$nota->getCdfilial(),$nota->getNrlanctonf(), $nrseqEvento,$dscorrecaonfe,
                                  $cdstatus,$dsstatus,$dataFinal, $protocolo, " ", $cdempresa);
                                  
                                } 
                                
                                $xmlCCe = $this->trataXML($this->nfeTools->lastRequest, $this->nfeTools->lastResponse,$evento["nSeqEvento"]);
                                $this->gravaXmlBanco($nota->getNrorg(), $xmlCCe, $nota->getCdfilial(),$nota->getNrlanctonf(), self::TIPO_ARQ_XML_CARTA_CORRECAO); 
                            }
                        }  
                    }
                    if($situacaoSefaz != $statusnfe){
                        //denega nota
                        $protocolo = $infoSefaz['aProt']['nProt'];
                        $dthoraprot = $infoSefaz['aProt']['dhRecbto'];
                        $motivo = $infoSefaz['aProt']['xMotivo'];
                        $novaNota = array();
                        $novaNota['CDFILIAL'] = $nota->getCdfilial();
                        $novaNota['NRLANCTONF'] = $nota->getNrlanctonf(); 
                        $novaNota['IDENTRSAIDOP'] = $nota->getIdentrsaidop(); 
                        $novaNota['NRORG'] = $nota->getNrorg();
                        
                        $resp = $this->gerarNovaNotaAposDenegacao($novaNota, $dthoraprot,$motivo);
                        $this->transmissaoNfeFactory->insereXmlCancelado($novaNota['CDFILIAL'], $novaNota['NRLANCTONF'], $resp['CDFILIAL'], $resp['NRLANCTONF']);
                        $this->transmissaoNfeFactory->removeXmlCancelado($novaNota['CDFILIAL'], $novaNota['NRLANCTONF']);
                        $xml = $this->transmissaoNfeFactory->verificaXml($nota->getCdfilial(),$nota->getNrlanctonf(), self::TIPO_ARQ_XML_DADOS_NOTA_FISCAL);
                    }
                    
                }else if($situacaoSefaz == "A"){
                    foreach ($infoSefaz['aEvent'] as $eventos) {
                        foreach ($eventos as $evento) {
                            if($evento["tpEvento"] == 110110){
                                $cdstatus = $evento["cStat"];
                                $dsstatus = $evento["xMotivo"];
                                $nrseqEvento = str_pad($evento["nSeqEvento"], 2, "0", STR_PAD_LEFT);
                                //$nrseqEvento = $evento["nSeqEvento"];
                                //$dataFinal = $this->constroiData(substr($evento["dhRegEvento"], 0, 10));
                                $dataFinal = $evento["dhRegEvento"];
                                $protocolo = $evento["nProt"];
                                $dscorrecaonfe = $evento['xCorrecao'];
                                
                                $carta = $this->transmissaoNfeFactory->verificaCartaCor($nota->getCdfilial(),$nota->getNrlanctonf(), $nrseqEvento);
                                if(!$carta){
                                  $this->insereCarta($nota->getNrorg(),$nota->getCdfilial(),$nota->getNrlanctonf(), $nrseqEvento,$dscorrecaonfe,
                                  $cdstatus,$dsstatus,$dataFinal, $protocolo, " ", $cdempresa);
                                  
                                } 
                                $xmlCCe = $this->trataXML($this->nfeTools->lastRequest, $this->nfeTools->lastResponse,$evento["nSeqEvento"]);
                                $this->gravaXmlBanco($nota->getNrorg(), $xmlCCe, $nota->getCdfilial(),$nota->getNrlanctonf(), self::TIPO_ARQ_XML_CARTA_CORRECAO); 
                            }
                        }  
                    }
                    //atualiza  dados de autorização
                    if(isset($infoSefaz['aProt']['nProt'])){
                        $protocolo = $infoSefaz['aProt']['nProt'];
                    }
                    if(isset($infoSefaz['aProt']['dhRecbto'])){
                        $data = $infoSefaz['aProt']['dhRecbto'];
                        $data = $this->constroiData(substr($data, 0, 10));
                    }
                    if(isset($infoSefaz['aProt']['xMotivo'])){
                        $dsstatus = $infoSefaz['aProt']['xMotivo'];
                    }
                    if($situacaoSefaz != $statusnfe){
                        $novaNota = array();
                        $novaNota['CDFILIAL'] = $nota->getCdfilial();
                        $novaNota['NRLANCTONF'] = $nota->getNrlanctonf(); 
                        $novaNota['IDENTRSAIDOP'] = $nota->getIdentrsaidop(); 
                        $novaNota['NRORG'] = $nota->getNrorg();
                    }
                    $this->transmissaoNfeFactory->atualizaDadosAutorizacaoDenegacao($nota->getCdfilial(),$nota->getNrlanctonf(), $situacaoSefaz,$dsstatus,$protocolo,$data);
                }
                return array('SUCESS' => 'Informações da nota sincronizadas com sucesso');
            }  
        }else{
            if($statusnfe == "T"){
                $this->transmissaoNfeFactory->atualizaSituacaoNota($chave,'R');
                if($isSincronizaLote){
                    return array('CHAVE' => $chave); 
                }else{
                   return array('ERRO' => $infoSefaz["cStat"]. " - ".$infoSefaz["xMotivo"], 'ATUALIZOU'=> 'TRUE'); 
                }
                
            }else{
                return array('ERRO' => $infoSefaz["cStat"]. " - ".$infoSefaz["xMotivo"], 'REJEICAO' => 'TRUE');
            }
        }
    }
    
    public function sincronizaLote($chave,$isAtualizada){
        $nota = self::getNotaFiscal($chave);
        if(empty($nota)){
            return array('ERRO' => 'Não foi possível atualizar Lote');
        }
        $situacaoLote = $this->transmissaoNfeFactory->consultaSituacaoLote($nota->getNrlotenfe());
        $chaves = array();
        $notasLote = $this->transmissaoNfeFactory->consultaNotasLote($nota->getNrlotenfe());
        if($situacaoLote['IDSTATUSLOTENFE'] == 'E' || $situacaoLote['IDSTATUSLOTENFE'] == 'T'){
             foreach($notasLote as $notaLote){
                $novaNota = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($notaLote['CDFILIAL'], $notaLote['NRLANCTONF'], $notaLote['NRORG']);
                //if($novaNota->getNracessonfe() != $chave){
                    $resp = $this->sincronizaDadosNota($novaNota->getNracessonfe(), true);
                    if(isset($resp['CHAVE'])){
                       array_push($chaves,$resp['CHAVE']); 
                    }
                    if(isset($resp['ERRO'])){
                        if($resp['ERRO'] == "Não foi possivel sincronizar. Tente novamente mais tarde."){
                            $this->atualizaSituacaoNotas($chave,$chaves,$isAtualizada);
                            return array('ERRO' => 'Não foi possível atualizar Lote');
                        }
                    }
            }
            $this->transmissaoNfeFactory->atualizaSituacaoLote($nota->getNrlotenfe());
            return array('SUCESS' => 'Atualizou lote');
        }else{
            return array('CONCLUIDO' => 'Lote já concluído');
        }
           
    }
    
    private function atualizaSituacaoNotas($chaveConsultada,$chaveLotes,$isAtualizada){
        foreach($chaveConsultada as $chave){
            $this->transmissaoNfeFactory->atualizaSituacaoNota($chave,'T');
        }
        if($isAtualizada){
            $this->transmissaoNfeFactory->atualizaSituacaoNota($chaveConsultada,'T');
        }
    }
    
    private function gerarNovaNotaAposDenegacao($nota,$dthoraprot,$motivo)
    {
        try {
            $notafiscalorigem = Notafiscal::findOneByCdfilialAndNrlanctonfAndIdentrsaidopAndNrorg(
                $nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP'], $nota['NRORG']);
            $this->formatDtHrProt($dthoraprot);
            
            //Insere a nova Nota Fiscal
            /** @var $novaNotafiscal NotaFiscal */
            $novaNotafiscal = clone $notafiscalorigem;
            $novaNotafiscal->setNrlanctonf(
                $this->novoCodigo->getCodeByTableName(
                    'LANCNF/FILIAL' . $notafiscalorigem->getCdfilial(), $notafiscalorigem->getNrorg())
            );
            $novaNotafiscal->setIdbloqueado(Notafiscal::NOTA_FISCAL_BLOQUEADA);
            $novaNotafiscal->setIdstatus(self::STATUS_NF_PENDENTE);
            $novaNotafiscal->setDtcancelanf($dthoraprot);
            // $novaNotafiscal->setCdopecancnf(Ambiente::getCdOperador());
            $novaNotafiscal->setIdstatusnfe(self::STATUS_NF_DENEGADA);
            $novaNotafiscal->setDsobsstatusnfe($motivo);
            $novaNotafiscal->persist();
            $this->entityManager->flush();
            
            //Updata manualmente a Nota Fiscal de Origem em alguns campos que não existem na entidade notafiscal.
            $this->transmissaoNfeFactory->updateNFeCancelEPEC($nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP'], $nota['NRORG']);
            
            //Updata a Nota Fiscal de Origem
            $notafiscalorigem->setNrnotafisc(null);
            $notafiscalorigem->setNracessonfe(null);
            $notafiscalorigem->setNrprotocolonfe(null);
            $notafiscalorigem->setIdstatusnfe(Notafiscal::PENDENTE);
            $notafiscalorigem->setNrnotafiscalele(null);
            $notafiscalorigem->setDsobsstatusnfe(null);
            $notafiscalorigem->setCdemplotenfe(null);
            $notafiscalorigem->setNrlotenfe(null);
            $notafiscalorigem->setIdbloqueado(Notafiscal::NOTA_FISCAL_NAO_BLOQUEADA);
            $notafiscalorigem->setIdstatus(Notafiscal::PENDENTE);
            $notafiscalorigem->setDtsaidemis(null);
            $notafiscalorigem->setDtemissao(null);
            $notafiscalorigem->setCdoperapro(null);
            $notafiscalorigem->setNrlanctbdoc(null);
            $notafiscalorigem->setDtexprnf(null);
            $notafiscalorigem->setNrexprnf(null);
            $notafiscalorigem->setIdtipoemisnfe(NotaFiscal::IDTIPOEMISNFE_NAO_EMITIDO);
            $notafiscalorigem->setIdemailnfe('P');
            $notafiscalorigem->setIdemailnfetransp('P');
            $notafiscalorigem->setIdtpambnfe(Notafiscal::IDTPAMBNFE_0);
            $notafiscalorigem->setCdbarraconting(null);
            $notafiscalorigem->setDthrprotocolnfe(null);
            $notafiscalorigem->setCdparamemisre(null);
            $notafiscalorigem->setNrseqcapadoc(null);
            $notafiscalorigem->setNrregdpecnfe(null);
            $notafiscalorigem->setDthrregcontnfe(null);
            $notafiscalorigem->persist();
           
            //Atualiza carta de correção caso exista
            $cartaCorrecao = Cartacornfe::findByNrorgAndCdfilialAndNrlanctonf($notafiscalorigem->getNrorg(), $notafiscalorigem->getCdfilial(), $notafiscalorigem->getNrlanctonf());
            foreach($cartaCorrecao as $carta){
                /** @var Cartacornfe $novaCartaCorrecao */
                $novaCartaCorrecao = clone $carta;
                $novaCartaCorrecao->setNrlanctonf($novaNotafiscal->getNrlanctonf());
                $novaCartaCorrecao->persist();
                $this->entityManager->remove($carta);
            }

            $this->entityManager->flush();
            return array('DTHRPROTOCOLNFE' => $dthoraprot->format('d/m/Y H:i:s'),
                    'DSOBSSTATUSNFE' => $motivo,
                    'CDFILIAL' => $novaNotafiscal->getCdfilial(),
                    'NRLANCTONF' => $novaNotafiscal->getNrlanctonf());
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function trataXML($xmlEnvio, $xmlRetorno, $nEvento){
        
        $xmlRetorno = explode("<retConsSitNFe", $xmlRetorno);
        $xmlRetorno = '<retConsSitNFe'.$xmlRetorno[1];
        $xmlRetorno = explode('</retConsSitNFe>', $xmlRetorno);
        $xmlRetorno = $xmlRetorno[0]."</retConsSitNFe>";
        
        
        $xmlRetorno = explode("</protNFe>", $xmlRetorno);
        $xml = $xmlRetorno[1];
        $xml= explode('</retConsSitNFe>', $xml);
        $xml = explode('</procEventoNFe>', $xml[0]);
        
        
        foreach($xml as $evento){
            $eventos = array();
            $eventos = explode("</infEvento>", $evento);
            $informações = array();
            $informações = explode("<nSeqEvento>", $eventos[0]);
            $sequencia = array();
            $sequencia = explode("</nSeqEvento>", $informações[1]);
            $nSeqEvento = $sequencia[0];
            if($nSeqEvento == $nEvento){
                $novoXml = array();
                $novoXml = explode("<procEventoNFe ", $evento);
                $evento = '<procEventoNFe xmlns="http://www.portalfiscal.inf.br/nfe"'. $novoXml[1]. "</procEventoNFe>";
                return $evento;
            }
        }
    }
    
    public function getTagXml($xml,$tag){
        $valor = explode("<".$tag.">", $xml);
        $novoValor = $valor[1];
        $valor = explode("</".$tag.">",$novoValor);
        return $valor[0];
    }
    
    public function constroiData($data){
        $dia = substr($data, -2);
        $mes = substr($data, 5, -3);
        $ano = substr($data, 0, 4);
        $dataFinal = $dia . "/" . $mes . "/" . $ano;
        return $dataFinal;
    }

    private function protocoloLoteProcessado($nrlotenfe, $nrrecibonfe, $retornoSefaz){
        $this->updLoteNfe($this->nfeTools->config->cdempresa, $nrlotenfe, self::LOTE_PROCESSADO, $this->aResposta['xMotivo']);

        foreach ($this->aResposta['aProt'] as $protocolo) {
            $nota = self::getNotaFiscal($protocolo['chNFe']);
            $consultaSituacao = array(false);

            if($protocolo["cStat"] == self::ID_NF_DUPLICIDADE || $protocolo["cStat"] == self::ID_NF_DUPLICIDADE_CHAVE_ACESSO_MAIS_DE_1X ||
               $protocolo["cStat"] == self::ID_NF_DUPLICIDADE_DIFERENCA_CHAVE_ACESSO) {
                   
                $consultaSituacao = $this->consultaSituacaoNF($nota, $this->nfeTools->config, false);
            }
            
            if(!$consultaSituacao[0]) {
                switch ($protocolo['cStat']) {
                    case (self::ID_NF_AUTORIZADA):
                    case (self::ID_NF_AUTORIZADA_FORA_DO_PRAZO):
                        //NFe aprovada
                        self::alteraNotaFiscal($nota,
                            $protocolo['chNFe'],
                            self::STATUS_NF_ACEITA,
                            $protocolo['xMotivo'],
                            $protocolo['nProt'],
                            $protocolo['dhRecbto']
                        );
                        $procXML = self::adicionaProtocolo($nota->getCdfilial(), $nota->getNrlanctonf(), $nrrecibonfe, $retornoSefaz);
                        $this->gravaXmlBanco($nota->getNrorg(), $procXML, $nota->getCdfilial(), $nota->getNrlanctonf(), self::TIPO_ARQ_XML_AUTORIZACAO);
                        
                        $this->notasLoteSuccess[] = ["CDFILIAL"        =>  $nota->getCdFilial(),
                                                     "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                                     "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                                     "NRACESSONFE"     =>  $nota->getNracessonfe(),
                                                     "NRORG"           =>  $nota->getNrorg()];
                        break;
                    case (self::ID_NF_DENEGADA):
                    case (self::ID_NF_DENEGADA_EMIT):
                    case (self::ID_NF_DENEGADA_DEST):
                        //NFe Denegada
                        self::alteraNotaFiscal($nota,
                            $protocolo['chNFe'],
                            self::STATUS_NF_DENEGADA,
                            $protocolo['xMotivo'],
                            $nrrecibonfe,
                            $protocolo['dhRecbto'],
                            self::DESBLOQUADO
                        );
                        // $this->salvaXmlNfe($this->envdenDir, $nota->getNracessonfe(), '', $this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
                        unlink($this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
                        break;
                    default:
                        //Nf Rejeitada
                        self::alteraNotaFiscal($nota,
                            $protocolo['chNFe'],
                            self::STATUS_NF_REJEITADA,
                            $protocolo['xMotivo'],
                            $nrrecibonfe,
                            $protocolo['dhRecbto'],
                            self::DESBLOQUADO
                        );
                        // $this->salvaXmlNfe($this->envrejDir, $nota->getNracessonfe(), '', $this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
                        // unlink($this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
                }
            }
        }
    }

    private function protocoloLoteEmProcessamento($nrlotenfe){
        $this->updLoteNfe($this->nfeTools->config->cdempresa, $nrlotenfe, self::LOTE_EM_PROCESSAMENTO, $this->aResposta['xMotivo']);
        $notasLote = Notafiscal::findByNrlotenfeAndCdemplotenfeAndNrorg($nrlotenfe, $this->nfeTools->config->cdempresa, $this->nfeTools->config->nrorg);
        /** @var Notafiscal $nota */
        foreach($notasLote as $nota){
            self::alteraNotaFiscalDif104(
                $nota,
                $this->nfeTools->config->cdempresa,
                $nrlotenfe,
                $this->nfeTools->config->nrorg,
                'T',
                null,
                '',
                $this->aResposta['cStat'],
                $this->aResposta['xMotivo'],
                self::BLOQUEADO
            );
        }
    }

    private function protocoloLoteNaoLocalizado($nrlotenfe){
        $this->updLoteNfe($this->nfeTools->config->cdempresa, $nrlotenfe, self::LOTE_NAO_LOCALIZADO, $this->aResposta['xMotivo']);
        $notasLote = Notafiscal::findByNrlotenfeAndCdemplotenfeAndNrorg($nrlotenfe, $this->nfeTools->config->cdempresa, $this->nfeTools->config->nrorg);
        /** @var Notafiscal $nota */
        foreach($notasLote as $nota){
            self::alteraNotaFiscalDif104(
                $nota,
                $this->nfeTools->config->cdempresa,
                $nrlotenfe,
                $this->nfeTools->config->nrorg,
                'P',
                null,
                '',
                $this->aResposta['cStat'],
                $this->aResposta['xMotivo'],
                self::DESBLOQUADO
            );
        }
    }

    private function protocoloProblemaReciboOuCnpj($nrlotenfe){
        $this->updLoteNfe($this->nfeTools->config->cdempresa, $nrlotenfe, self::PROBLEMA_RECIBO_OU_CNPJ, $this->aResposta['xMotivo']);
        $ambiente = $this->nfeTools->config->tpAmb == '1' ? 'producao' : 'homologacao';
        
        $notasLote = Notafiscal::findByNrlotenfeAndCdemplotenfeAndNrorg($nrlotenfe, $this->nfeTools->config->cdempresa, $this->nfeTools->config->nrorg);
        /** @var Notafiscal $nota */
        foreach($notasLote as $nota){
            self::alteraNotaFiscalDif104(
                $nota,
                $this->nfeTools->config->cdempresa,
                $nrlotenfe,
                $this->nfeTools->config->nrorg,
                self::STATUS_NF_REJEITADA,
                null,
                '',
                $this->aResposta['cStat'],
                $this->aResposta['xMotivo'],
                self::DESBLOQUADO
            );
            // $this->salvaXmlNfe($this->rejDir, $nota->getNracessonfe(), '', $this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
        }
    }

    private function updLoteNfe($cdempresa, $nrlote, $status, $situacao)
    {
        $lotenfe = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Lotenfe')->findOneBy(
                array('cdemplotenfe' => $cdempresa, 'nrlotenfe' => $nrlote));
        $lotenfe->setIdstatuslotenfe($status);
        $lotenfe->setDssitlotenfe($situacao);
        $this->entityManager->persist($lotenfe);
        $this->entityManager->flush();
    }

    private function adicionaProtocolo($cdfilial, $nrlanctonf, $recibo, $retornoSefaz)
    {
        $xmlEnvio = $this->transmissaoNfeFactory->getXMLEnvio($cdfilial, $nrlanctonf, 'D');
        
        $domXml = new \DOMDocument("1.0", "UTF-8");
		$domXml->loadXML($retornoSefaz, LIBXML_NOBLANKS);
    
        return $this->addProtocolo($xmlEnvio[0]["DSARQXMLNFE_CLOB"], $retornoSefaz);
        // unlink($pathNFefile);
        // unlink($pathProtfile);
    }

    /**
     * @param $chaveDeAcesso
     * @return $notafiscal
     */
    private function getNotaFiscal($chaveDeAcesso)
    {
        /**@var  NotaFiscal */
        $notafiscal = Notafiscal::findOneByNracessonfe($chaveDeAcesso);
        return $notafiscal;
    }

    private function enviarEmailProtocolo($nota, $temErroEmail){
        try {
            $enviaParaTransportadora = ($nota->getNrinsjurtran()) ? true : false;
        
            $notaParaEnvio = array("NRACESSONFE" => $nota->getNracessonfe(), 
                                   "DTEMISSAO"   => $nota->getDtemissao(),
                                   "CDFILIAL"    => $nota->getCdfilial(),
                                   "NRLANCTONF"  => $nota->getNrlanctonf(),
                                   "CDEMPRESA"   => $nota->getCdempresa());
            
            $emailEnviado = $this->enviaEmail($notaParaEnvio);
            
            if($emailEnviado) {
                $this->sucessoEmail[] = array("CDFILIAL"        =>  $nota->getCdfilial(),
                                              "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                              "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                              "NRORG"           =>  $nota->getNrorg());
                $this->atualizaStatusEmailNF($notaParaEnvio, 'C');
            } else {
                $temErroEmail = true;
                
                $this->errosEmail[] = array("CDFILIAL"        =>  $nota->getCdfilial(),
                                            "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                            "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                            "NRORG"           =>  $nota->getNrorg(),
                                            "TPERRO"          =>  "E",
                                            "DSERRO"          =>  "Tentativa envio e-mail normal: ".$emailEnviado);
            }
            if($enviaParaTransportadora) {
                $emailEnviadoTransp = $this->enviaEmail($notaParaEnvio, 'T');
                
                if($emailEnviadoTransp) {
                    $this->sucessoEmail[] = array("CDFILIAL"        =>  $nota->getCdfilial(),
                                                  "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                                  "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                                  "NRORG"           =>  $nota->getNrorg());
                    $this->atualizaStatusEmailNF($notaParaEnvio, 'T');
                } else {
                    $temErroEmail = true;
                    
                    $this->errosEmail[] = array("CDFILIAL"        =>  $nota->getCdfilial(),
                                                "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                                "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                                "NRORG"           =>  $nota->getNrorg(),
                                                "TPERRO"          =>  "E",
                                                "DSERRO"          =>  "Tentativa envio e-mail transportadora: ".$emailEnviadoTransp);
                }
            }
            return $temErroEmail;
        } catch (\Exception $e) {
            $temErroEmail = true;
            
            $this->errosEmail[] = array("CDFILIAL"        =>  $nota->getCdfilial(),
                                        "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                        "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                        "NRORG"           =>  $nota->getNrorg(),
                                        "TPERRO"          =>  "E",
                                        "DSERRO"          =>  $e->getMessage());
            return $temErroEmail;
        }
    }

    private function alteraNotaFiscal($notafiscal, $chaveDeAcesso, $idstatusnfe, $dsobsstatusnfe, $nrprotocolonfe, $dthrprotocolnfe, $idbloq = null)
    {
        try {
            if ($notafiscal) {
                $notafiscal->setIdstatusnfe($idstatusnfe);
                $notafiscal->setDsobsstatusnfe($dsobsstatusnfe);
                $notafiscal->setNrprotocolonfe($nrprotocolonfe);
                $dthrprotocolnfe = date_create($dthrprotocolnfe);
                $notafiscal->setDthrprotocolnfe($dthrprotocolnfe);
                if ($idbloq) {
                    $notafiscal->setIdbloqueado($idbloq);
                }
                $notafiscal->persist();
                
                /** @var  $relacnflotenfe Relacnflotenfe */
                $relacnflotenfe = Relacnflotenfe::findOneByCdfilialAndNrlanctonfAndNrorgAndNrlotenfe($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg(), $notafiscal->getNrlotenfe());
                $relacnflotenfe->setIdstatusrelnfe($idstatusnfe);
                $relacnflotenfe->persist();
                
                $this->entityManager->flush();
            } else {
                throw new \Exception("Nota Fiscal não Encontrada pela chave de acesso: {$chaveDeAcesso}.");
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function alteraNotaFiscalDif104($notafiscal, $cdempresa, $nrlotenfe, $nrorg, $idstatusnfe, $dataProtocolo, $nrProtocolo, $cdstatus, $dsstatus, $idbloq = null)
    {
        try {
            if ($notafiscal instanceof Notafiscal) {
                $notafiscal->setIdstatusnfe($idstatusnfe);
                $notafiscal->setDsobsstatusnfe($dsstatus);
                if ($idbloq) {
                    $notafiscal->setIdbloqueado($idbloq); 
                }
                $notafiscal->persist();

                /** @var  $relacnflotenfe Relacnflotenfe */
                $relacnflotenfe = Relacnflotenfe::findOneByCdemplotenfeAndNrlotenfeAndNrorg($cdempresa, $nrlotenfe, $nrorg);
                $relacnflotenfe->setDtprotocolrelnfe($dataProtocolo);
                $relacnflotenfe->setNrprotocolrelnfe($nrProtocolo);
                $relacnflotenfe->setIdstatusrelnfe($idstatusnfe);
                $relacnflotenfe->setCdstatusrelnfe($cdstatus);
                $relacnflotenfe->setDsstatusrelnfe($dsstatus);
                $relacnflotenfe->persist();

                $this->entityManager->flush();
            } else {
                throw new \Exception("Nota fiscal não encontrada");
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /*private function formatData($data)
    {
        try {
            if ($data instanceof \Setforms2\Shared\ORM\DateTime === false) {
                $data = DateUtil::getDataDeString($data);
            }
            return $data;
        } catch (\Exception $e) {
            throw $e;
        }
    }*/

    public function montaArrayRetorno(Notafiscal $nota){
        $this->aRetorno[$nota->getNrlanctonf()] =
            array('DTEMISSAO' => $nota->getDtemissao()->format('d/m/Y'),
                  'NRACESSONFE' => $nota->getNracessonfe(),
                  'NRNOTAFISC' => $nota->getNrnotafisc(),
                  'IDSTATUSNFE' => $nota->getIdstatusnfe(),
                  'NRPROTOCOLONFE' => $nota->getNrprotocolonfe(),
                  'NRNOTAFISCALELE' => $nota->getNrnotafiscalele());
    }
    
    private function sefazStatus($uf, $cUF, $tpAmb, $datahoracerta, $checkContingency = false){
        
        if($checkContingency) {
            $this->aRespostaContingency = [];
            $this->nfeTools->contingency->activate($uf, 'Consulta status de contingência.');
            $uf = '';
        }
        else $this->aResposta = [];
        
        $method   = !$checkContingency ? 'saveStatServ' : 'saveStatServContingency';
        $variable = !$checkContingency ? 'aResposta'    : 'aRespostaContingency';
        
        $resp = $this->nfeTools->sefazStatus($uf, $tpAmb);
        $this->$variable = ReturnNFe::readReturnSefaz('NfeStatusServico', $resp);
        
        if($resp)
            $this->transmissaoNfeFactory->$method($this->$variable['cStat'],
                                                  substr($this->$variable['xMotivo'], 0, 255),
                                                  substr($this->$variable['xObs'], 0, 255),
                                                  date_format(date_create($this->$variable['dhRecbto']), 'd/m/Y H:i:s'),
                                                  $cUF, $tpAmb);
        
        else $this->transmissaoNfeFactory->$method(999, 'Erro Não Catalogado', null, $datahoracerta, $cUF, $tpAmb);
        
        if($checkContingency) $this->nfeTools->contingency->deactivate();
        
        return $resp;
    }

    private function enviaLote(){
        $tpAmb = $this->nfeTools->config->tpAmb;
        $xmls = [];
        $resp = $this->nfeTools->sefazEnviaLote($this->aXml, $this->idLote, 0, false, $xmls);
        
        if(!empty($xmls)) $this->aXml = $xmls;
        
        $this->aResposta = ReturnNFe::readReturnSefaz('NfeAutorizacao', $resp);
        
        return $resp;
    }
    
    private function enviaLoteEPEC(){
        $this->nfeTools->setVerAplic('4.0.50'); //versão do aplicativo emissor
        $resp = $this->nfeTools->sefazEPEC($this->aXml[0]); //Biblioteca só transmite um lote com uma nota epec por vez.
        
        $this->aResposta = ReturnNFe::readReturnSefaz('RecepcaoEPEC', $resp);
        
        return $resp;
    }
    
    private function insereLoteNfe(){
        try {
            $lotenfe = new Lotenfe();
            $lotenfe->setNrlotenfe($this->idLote);
            $lotenfe->setCdemplotenfe($this->nfeTools->config->cdempresa);
            $lotenfe->setNrorg($this->nfeTools->config->nrorg);
            $lotenfe->setNrrecibonfe($this->aResposta['nRec']);
            $lotenfe->setIdtpamblotenfe($this->nfeTools->config->tpAmb);
            $lotenfe->setIdtipoemislote($this->tpEmis);
            $lotenfe->setIdstatuslotenfe(self::LOTE_EM_PROCESSAMENTO);
            $lotenfe->setDtenviolotenfe(DateUtil::getDataAtual());
            $this->entityManager->persist($lotenfe);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Updata os dados das notas fiscais enviadas (Status, Chave de Acesso, Nº do Lote)
     *
     */

    private function updateNotaFiscal(Notafiscal $nota, $tipo)
    {
        try {
            if(!$tipo || $tipo == 'SVC') {
                $nota->setIdstatusnfe(self::STATUS_NF_TRANSMITIDA);
                $nota->setNrlotenfe($this->idLote);
                $nota->setCdemplotenfe($nota->getCdempresa());
                
                $dtprocessa = $this->nfeTools->config->dadosFilial->DTPROCESSA;
                if ($dtprocessa) {
                    $nota->setDtaprovacao(DateUtil::getDataDeString($dtprocessa));
                }
            }
            else if($tipo == 'FSDA' || $tipo == 'EPEC') {
                $nota->setIdstatusnfe(self::STATUS_NF_PENDENTE);
                $nota->setDsobsstatusnfe(null);
                $nota->setNrprotocolonfe(null);
                $nota->setDthrprotocolnfe(null);
            }
            $nota->setIdbloqueado(self::BLOQUEADO);
            $nota->setIdtpambnfe($this->nfeTools->config->tpAmb);
            $nota->setIdtipoemisnfe($this->tpEmis);
            $nota->setNrnotafisc($nota->getNrnotafiscalele());
            $nota->persist();
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function updateNotaOl(Notafiscal $nota) {
        $dtEmissao = $nota->getDtemissao()->format('d-m-Y H:i:s');
        $this->transmissaoNfeFactory->updateNotaOl($nota->getNrnotafiscalele(), $dtEmissao, $nota->getCdfilial().$nota->getNrlanctonf());
    }
    
    private function updateNotaReplicada(Notafiscal $nota) {
        $dtEmissao = $nota->getDtemissao()->format('d-m-Y H:i:s');
        $this->transmissaoNfeFactory->updateNotaReplicada($dtEmissao, $nota->getNrnotafiscalele(), $nota->getCdfilial().$nota->getNrlanctonf(), $nota->getCdfilirece(), $nota->getCdfiliemit());
    }

    private function insereCarta($nrorg, $cdfilial, $nrlanctonf, $nrseq, $descricao,
                                       $status, $motivo, $dthrprotocolo,$protocolo, $numlote, $emprelote,
                                       $idemailcce = 'P', $idemailccetrans = 'N' ){
        /** @var $cartacorr Cartacornfe */
        $cartacorr = new Cartacornfe();
        $cartacorr->setNrorg($nrorg);
        $cartacorr->setCdemplotecce($emprelote);
        $cartacorr->setCdfilial($cdfilial);
        $cartacorr->setNrlanctonf($nrlanctonf);
        $cartacorr->setNrseqcce($nrseq);
        $cartacorr->setDscorrecaonfe($descricao);
        $cartacorr->setCdstatuscce($status);
        $cartacorr->setDsobsstatuscce($motivo);
        $cartacorr->setDthrprotocolcce(DateUtil::getDataDeString(substr($dthrprotocolo, 0, 10), 'Y-m-d'));
        $cartacorr->setNrprotocolcce($protocolo);
        $cartacorr->setNrlotecce($numlote);
        $cartacorr->setIdemailcce($idemailcce);
        $cartacorr->setIdemailccetrans($idemailccetrans);
        $cartacorr->persist();
        $this->entityManager->flush();
    }

    private function insereRelacnflotenfe($notafiscal)
    {
        $relacnflotenfe = new Relacnflotenfe();
        $relacnflotenfe->setIdstatusrelnfe(self::STATUS_NF_TRANSMITIDA);
        $relacnflotenfe->setNrrelnflotenfe($this->novoCodigo->getCodeByTableName(
                    'RELACLOTENFE', $notafiscal->getNrorg()));
        $relacnflotenfe->setCdemplotenfe($notafiscal->getCdempresa());
        $relacnflotenfe->setNrlotenfe($this->idLote);
        $relacnflotenfe->setCdfilial($notafiscal->getCdfilial());
        $relacnflotenfe->setNrorg($notafiscal->getNrorg());
        $relacnflotenfe->setNrlanctonf($notafiscal->getNrlanctonf());
        $relacnflotenfe->setNrnotafiscalele($notafiscal->getNrnotafisc());
        $relacnflotenfe->setIdambrelotenfe($this->nfeTools->config->tpAmb);
        $relacnflotenfe->setNracessorelnfe($this->chave);
        $relacnflotenfe->setCdfiliemit($notafiscal->getCdfiliemit());
        $relacnflotenfe->setCdinscesta($notafiscal->getCdinscesta());
        $relacnflotenfe->setCdinscmuni($notafiscal->getCdinscmuni());
        $relacnflotenfe->setCdserie($notafiscal->getCdserie());
        $relacnflotenfe->setCdserienfe($this->corpoNF['CDSERIE']);
        $relacnflotenfe->setDtenviolotenfe(DateUtil::getDataAtual());
        $relacnflotenfe->setCdfilirece($notafiscal->getCdfilirece());
        $relacnflotenfe->setVrnotafisc($notafiscal->getVrnotafisc());
        $relacnflotenfe->setDtemissao($notafiscal->getDtemissao());
        $relacnflotenfe->setIdentrsaidop($notafiscal->getIdentrsaidop());
        $relacnflotenfe->persist();
        $this->entityManager->flush();
    }

    private function geraVencimento($notafiscal){
		if($notafiscal->getIddocfinanc() != 'L'){
			$tipoOperacao = Tipooperac::findOneByCdtipooperAndIdentrsaidopAndNrorg($notafiscal->getCdtipooper(), $notafiscal->getIdentrsaidop(), $notafiscal->getNrorg());
			if($tipoOperacao->getIdutilvenc() != 'N'){
			    
			    $filiemit = $notafiscal->getIdentrsaidop() === 'E' ? $notafiscal->getCdfilirece() : $notafiscal->getCdfiliemit();
				try {
    				$this->proceduresNfFactory->procedureGeraDadosFinanceiro($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), 'T', $filiemit);
    				$this->entityManager->flush();
				} catch (\Exception $e) {
                    $msgError = $e->getMessage();
                    $cdCLASSFINAError = strpos($msgError, 'CDCLASSFINA');
                    
                    if($cdCLASSFINAError !== false) 
                        throw new \Exception('Tipo de conta a pagar não parametrizado.');
                    else 
                        throw new \Exception('Erro ao gerar dados financeiro. Exceção: '.$e);
                }
			}
		}
	}

    private function gravaNumeroNotafiscalEle(){
        $this->nota->setNrnotafiscalele($this->numeroNotaFiscal);
        $this->nota->persist();
        $this->entityManager->flush();
    }

    /**
     * Grava o arquivo xml no banco de dados (Tabela ARQXMLNFE)
     * @param $nrorg
     * @param $xml Strring
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $idtipoarqxml
     * @throws \Exception
     */
    public function gravaXmlBanco($nrorg, $xml, $cdfilial, $nrlanctonf, $idtipoarqxml)
    {
        try {
            $xml = (string)str_replace('<?xml version="1.0" encoding="UTF-8"?>', ' ', $xml);
            $xml = str_replace("\n", '', $xml);

            $arqxml = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Arqxmlnfe')->findOneBy(
                array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrlanctonf' => $nrlanctonf, 'idtipoarqxml' => $idtipoarqxml));

            if ($arqxml) {
                $arqxml->setDsarqxmlnfeClob($xml);
            } else {
                $arqxml = new Arqxmlnfe();
                $arqxml->setNrorg($nrorg);
                $arqxml->setCdfilial($cdfilial);
                $arqxml->setNrlanctonf($nrlanctonf);
                $arqxml->setIdtipoarqxml($idtipoarqxml);
                $arqxml->setDsarqxmlnfeClob($xml);
            }
            // $arqxml->persist();
            $this->entityManager->persist($arqxml);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Grava o arquivo xml no banco de dados (Tabela ARQXMLNFE)
     * @param $nrorg
     * @param $xml Strring
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $idtipoarqxml
     * @throws \Exception
     */
    public function gravaXmlBancoCancelado($nrorg, $xml, $cdfilial, $nrlanctonf, $cdFilialNovaNota, $nrLanctonfNovaNota)
    {
        try {
            $xml = (string)str_replace('<?xml version="1.0" encoding="UTF-8"?>', ' ', $xml);
            $xml = str_replace("\n", '', $xml);

            $this->transmissaoNfeFactory->insereXmlCancelado($cdfilial, $nrlanctonf, $cdFilialNovaNota, $nrLanctonfNovaNota);
            $this->transmissaoNfeFactory->removeXmlCancelado($cdfilial, $nrlanctonf);
            
            $arqxml = new Arqxmlnfe();
            $arqxml->setNrorg($nrorg);
            $arqxml->setCdfilial($cdFilialNovaNota);
            $arqxml->setNrlanctonf($nrLanctonfNovaNota);
            $arqxml->setIdtipoarqxml('C');
            $arqxml->setDsarqxmlnfeClob($xml);

            $this->entityManager->persist($arqxml);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMaxNrseqcce($filial, $lancamento){
        return $this->transmissaoNfeFactory->getMaxNrseqcce($filial, $lancamento);
    }

    private function formatDtHrProt(&$dthoraprot)
    {
        $dthoraprot = substr($dthoraprot, 8, 2) . '/' . substr($dthoraprot, 5, 2) . '/' . substr($dthoraprot, 0, 4) . ' ' . substr($dthoraprot, 11, 8);
        $dthoraprot = DateUtil::getDataDeString($dthoraprot, "d/m/Y H:i:s");
    }

    /**
     * @param $aMailConf
     */
    private function retirarArrobaEmail($aMailConf)
    {
        return str_replace("@", "&#64", $aMailConf);
    }

    /**
     * @param $config
     * @return mixed|string
     */
    private function recolocarArrobaEmail($config)
    {
        $config            = json_decode($config);
        $config->aMailConf = json_decode(str_replace("&#64", "@", json_encode($config->aMailConf)));
        $config            = json_encode($config);
        return $config;
    }
    
	public function decrypt($text, $salt = 'teknisa') {
          return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    /**
     * @param $salt
     * @return string
     */
    protected function padSalt($salt) {
        //Only keys of sizes 16, 24 or 32 supported.
        //http://php.net/manual/pt_BR/function.mcrypt-encrypt.php See changelog.
        $salt = str_pad($salt, 32, "\0");
        return $salt;
    }
    
    public function getToolsNFe($ie, $cnpj, $config) {
        
        $certificado = $this->transmissaoNfeFactory->getCertificado($ie, $cnpj, $config->cdempresa);

    	$certificadoEncontrado = false;
    	
    	foreach($certificado as $cert) {
    	    if($cert["DSDIRCERTIFICADO"]) {
    	        $certificadoEncontrado = true;
    	        $certify = $cert;
    	    }
    	}

		
	    if($certificadoEncontrado) {
            $senha = $this->decrypt($certify["CDSENHACERTIF"]);
    
    		$certificado = $certify["DSDIRCERTIFICADO"];
    		
    		$certificado = explode('>', $certificado);
    		
            $certificado = file_get_contents($this->fileServerLink.$certificado[1]);
	    } else {
	       throw new \Exception("Certificado Digital não encontrado.");
	    }
	    
        try {
            $certificado = Certificate::readPfx($certificado, $senha);
            $this->certificate = $certificado;
        }
        catch(\Exception $e) {
            throw new \Exception("Senha do certificado incorreta!");
        }
        
        $tools  = new NFe(json_encode($config), $certificado);
        $tools2 = new SoapCurl($certificado);
        $tools2->httpVersion('1.1');
        
        $tools->loadSoapClass($tools2);
			
        return $tools;
	}
	
	/**
     * Checks if given data is file, handles mixed input
     * @param  mixed $value
     * @return boolean
     */
    private function isFile($value)
    {
        $value = strval(str_replace("\0", "", $value));
        return is_file($value);
    }
	
	/**
     * Render a template with valid data
     * @param string $template
     * @param string $destinatario
     * @param string $data
     * @param string $numero
     * @param string $valor
     * @param string $chave
     * @param string $correcao
     * @param string $conduso
     * @return string
     */
    protected function renderTemplate(
        $template,
        $destinatario = '',
        $data = '',
        $numero = '',
        $valor = 0,
        $chave = '',
        $correcao = '',
        $conduso = '',
        $emitente = ''
    ) {
        $dt = new \DateTime(str_replace('T', ' ', $data));
        $search = array(
            '{destinatario}',
            '{data}',
            '{numero}',
            '{valor}',
            '{emitente}',
            '{chave}',
            '{correcao}',
            '{conduso}'
        );
        $replace = array(
          $destinatario,
          $dt->format('d/m/Y'),
          $numero,
          number_format($valor, 2, ',', '.'),
          $this->nfeTools->config->aMailConf->fantasy,
          $chave,
          $correcao,
          $conduso
        );
        $template = str_replace($search, $replace, $template);
        return $template;
    }
    
    public function loadDocuments($xml, $pdf = '', $config)
    {
        $this->xml = $xml;
        $this->pdf = $pdf;
        if ($this->isFile($xml)) {
            $this->xml = file_get_contents($xml);
        }
        if ($this->isFile($pdf)) {
            $this->pdf = file_get_contents($pdf);
        }
        //get xml data
        $this->getXmlData($this->xml, $config);
    }
    
    /**
     * Search xml for data
     * @param string $xml
     * @throws \InvalidArgumentException
     */
    protected function getXmlData($xml, $config)
    {
        $dom = new \DOMDocument('1.0', 'UTF-8');
        $dom->preserveWhiteSpace = false;
        $dom->loadXML($xml);
        $root = $dom->documentElement;
        $name = $root->tagName;
        $dest = $dom->getElementsByTagName('dest')->item(0);
        $ide = $dom->getElementsByTagName('ide')->item(0);

        switch ($name) {
            case 'nfeProc':
            case 'NFe':
                $type = 'NFe';
                $this->fields->numero = str_pad($ide->getElementsByTagName('nNF')->item(0)->nodeValue, 9, '0', STR_PAD_LEFT);
                $this->fields->valor = $dom->getElementsByTagName('vNF')->item(0)->nodeValue;
                $this->fields->data = $ide->getElementsByTagName('dhEmi')->item(0)->nodeValue;
                $this->fields->chave = $dom->getElementsByTagName('chNFe')->item(0)->nodeValue;
                $this->subject = "Emissão de Nota Fiscal Eletrônica. Chave: ". $dom->getElementsByTagName('chNFe')->item(0)->nodeValue;
                break;
            case 'cteProc':
            case 'CTe':
                $type = 'CTe';
                $this->fields->numero = str_pad($ide->getElementsByTagName('nCT')->item(0)->nodeValue, 9, '0', STR_PAD_LEFT);
                $this->fields->valor = $dom->getElementsByTagName('vRec')->item(0)->nodeValue;
                $this->fields->data = $ide->getElementsByTagName('dhEmi')->item(0)->nodeValue;
                $this->subject = "CTe n. " . $this->fields->numero . " - " . $config->fantasy;
                break;
            case 'procEventoNFe':
            case 'procEventoCTe':
                $type = 'CCe';
                $this->fields->chave = str_pad($dom->getElementsByTagName('chNFe')->item(0)->nodeValue, 9, '0', STR_PAD_LEFT);
                $this->fields->data = $dom->getElementsByTagName('dhEvento')->item(0)->nodeValue;
                $this->fields->correcao = $dom->getElementsByTagName('xCorrecao')->item(0)->nodeValue;
                $this->fields->conduso = $dom->getElementsByTagName('xCondUso')->item(0)->nodeValue;
                if (empty($this->fields->chave)) {
                    $this->fields->chave = $dom->getElementsByTagName('chCTe')->item(0)->nodeValue;
                }
                $this->subject = "Carta de Correção " . $config->fantasy;
                break;
            default:
                $type = '';
        }
        //get email adresses from xml, if exists
        //may have one address in <dest><email>
        if (!empty($dest)) {
            $this->fields->destinatario = $dest->getElementsByTagName('xNome')->item(0)->nodeValue;
            $email = !empty($dest->getElementsByTagName('email')->item(0)->nodeValue) ?
                $dest->getElementsByTagName('email')->item(0)->nodeValue : '';
        }
        if (!empty($email)) {
            $this->addresses[] = $email;
        }
        //may have others in <obsCont xCampo="email"><xTexto>fulano@yahoo.com.br</xTexto>
        $obs = $dom->getElementsByTagName('obsCont');
        foreach ($obs as $ob) {
            if (strtoupper($ob->getAttribute('xCampo')) === 'EMAIL') {
                $this->addresses[] = $ob->getElementsByTagName('xTexto')->item(0)->nodeValue;
            }
        }
        //xml may be a NFe or a CTe or a CCe nothing else
        if ($type != 'NFe' && $type != 'CTe' && $type != 'CCe') {
            $msg = "Você deve passar apenas uma NFe ou um CTe ou um CCe. "
                    . "Esse documento não foi reconhecido.";
            throw new \InvalidArgumentException($msg);
        }
    }
    
    public function verificaTipoOperFili($cdtipooper, $cdfilial, $identrsaidop) {
        $nrOrg = $this->environment->getNrOrgTrab(); 
        $nmOrg = $this->transmissaoNfeFactory->getOrganizacao($nrOrg)["NMORGANIZACAO"];
        if($nmOrg == 'GRSA - BRASIL') {
            $parambloqnfafil = $this->transmissaoNfeFactory->verificaTipoOperac($cdtipooper, $cdfilial, $identrsaidop);
            if(isset($parambloqnfafil[0])) {
                if($parambloqnfafil[0]["IDLIBERATPOPFIL"] == 'S') return true;
                else return false;
            }
            else return true;
        }
        else return true;
    }
    
    public function consultaSituacaoNF($nota, $config, $consultaDireta) {
        try{
            if(gettype($nota) != 'object') {
                $nota = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['NRORG']);
            }
            
            $config = json_encode($config);
            $config = $this->recolocarArrobaEmail($config);
            $config = json_decode($config);
            $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
            $retornoSefaz = $this->nfeTools->sefazConsultaChave($nota->getNracessonfe(), $config->tpAmb, $this->aResposta);
            
            $this->aResposta = ReturnNFe::readReturnSefaz('NfeConsultaProtocolo', $retornoSefaz);
            
            if(!empty($this->aResposta)) {
                if(empty($this->aResposta['aProt'])) {
                       if($consultaDireta) {
                           if($nota->getIdstatusnfe() == 'T') {
                               $this->protocolaNotaConsulta($nota, $this->aResposta, $retornoSefaz, $consultaDireta);
                               return array('SUCCESS' => $retornoSefaz, 'NOTAS_SUCCESS' => []);
                           }
                           else return array('ERRORS' => $this->aResposta['xMotivo']);
                       }
                       else return array(false);
                }
                else {
                    $this->protocolaNotaConsulta($nota, $this->aResposta['aProt'], $retornoSefaz, $consultaDireta);
                    
                    if($consultaDireta) return array('SUCCESS' => $retornoSefaz, 'NOTAS_SUCCESS' => $this->notasLoteSuccess);
                    else                return array(true);
                }
            }
            if($consultaDireta) return array('ERRORS' => 'Não foi possivel realizar a consulta. Tente novamente mais tarde.');
            else                return array(false);
        } catch (\Exception $e) {
            if($consultaDireta)     return array('ERRORS' => $e->getMessage());
            else                    return array(false);
        }
    }
    
    protected function protocolaNotaConsulta($nota, $protocolo, $retornoSefaz, $consultaDireta) {
        if($consultaDireta) {
            $notasRelac = Relacnflotenfe::findByCdemplotenfeAndNrlotenfeAndIdstatusrelnfeAndNrorg(
                $nota->getCdemplotenfe(), $nota->getNrlotenfe(), self::STATUS_NF_TRANSMITIDA ,$nota->getNrorg());
                
            if(count($notasRelac) == 1) {
                $this->updLoteNfe($nota->getCdemplotenfe(), $nota->getNrlotenfe(), self::PROBLEMA_RECIBO_OU_CNPJ, $this->aResposta['xMotivo']);
            }
        }
        switch ($protocolo['cStat']) {
            case (self::ID_NF_AUTORIZADA):
            case (self::ID_NF_AUTORIZADA_FORA_DO_PRAZO):
                //NFe aprovada
                self::alteraNotaFiscal($nota,
                    $protocolo['chNFe'],
                    self::STATUS_NF_ACEITA,
                    $protocolo['xMotivo'],
                    $protocolo['nProt'],
                    $protocolo['dhRecbto']
                );
                $procXML = self::adicionaProtocolo($nota->getCdfilial(), $nota->getNrlanctonf(), '', $retornoSefaz);
                $this->gravaXmlBanco($nota->getNrorg(), $procXML, $nota->getCdfilial(), $nota->getNrlanctonf(), self::TIPO_ARQ_XML_AUTORIZACAO);
                
                $this->notasLoteSuccess[] = ["CDFILIAL"        =>  $nota->getCdFilial(),
                                             "NRLANCTONF"      =>  $nota->getNrlanctonf(),
                                             "IDENTRSAIDOP"    =>  $nota->getIdentrsaidop(),
                                             "NRACESSONFE"     =>  $nota->getNracessonfe(),
                                             "NRORG"           =>  $nota->getNrorg()];
                break;
            case (self::ID_NF_DENEGADA):
            case (self::ID_NF_DENEGADA_EMIT):
            case (self::ID_NF_DENEGADA_DEST):
                //NFe Denegada
                self::alteraNotaFiscal($nota,
                    $protocolo['chNFe'],
                    self::STATUS_NF_DENEGADA,
                    $protocolo['xMotivo'],
                    '',
                    $protocolo['dhRecbto'],
                    self::DESBLOQUADO
                );
                // $this->salvaXmlNfe($this->envdenDir, $nota->getNracessonfe(), '', $this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
                unlink($this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
                break;
            default:
                //Nf Rejeitada
                self::alteraNotaFiscal($nota,
                    $protocolo['chNFe'],
                    self::STATUS_NF_REJEITADA,
                    $protocolo['xMotivo'],
                    '',
                    $protocolo['dhRecbto'],
                    self::DESBLOQUADO
                );
                // $this->salvaXmlNfe($this->envrejDir, $nota->getNracessonfe(), '', $this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
                // unlink($this->envDir . DIRECTORY_SEPARATOR . "{$nota->getNracessonfe()}-nfe.xml");
        }
    }
    
    public function gerarXmlNfe($notas) {
        $size = 20;
        $seed    = time();
        $zipname = sys_get_temp_dir().'/'.substr(sha1($seed), 40 - min($size,40)).'.zip';
        $zip        = new \ZipArchive;
        
        if(is_file($zipname))
            $zip->open($zipname, \ZipArchive::OVERWRITE);
        else
            $zip->open($zipname, \ZipArchive::CREATE);
                
        foreach($notas as $nota) {
            try{
                if($nota["IDSTATUSNFE"] !== 'C') $nomeArquivo = $nota["NRACESSONFE"].'-nfe.xml';
                else                             $nomeArquivo = $nota["NRACESSONFE"].'-can.xml';
                
                if($nota["IDSTATUSNFE"] == 'A' || $nota["IDTIPOEMISNFE"] == 4) { //Libera xml para nfe cancelada, aceita ou EPEC.
                    $xml = $this->getXML($nota, 'A');
                    $dir_file = sys_get_temp_dir().'/'.$nomeArquivo;
                }
                else if($nota["IDSTATUSNFE"] == 'C') {
                    $xml = $this->getXMLCancelado($nota);
                    $dir_file = sys_get_temp_dir().'/'.$nomeArquivo;
                }
                file_put_contents($dir_file, $xml);
                $zip->addFile($dir_file, $nomeArquivo);
                
            }catch(\Exception $e) {
            }
        }
        $zip->close();
        
        foreach($notas as $nota) {
            if(is_file(sys_get_temp_dir().'/'.$nomeArquivo)) unlink(sys_get_temp_dir().'/'.$nomeArquivo);
        }
        return $zipname;
    }
    
        /**
     * addProtocolo
     * Adiciona o protocolo de autorização de uso da NFe
     * NOTA: exigência da SEFAZ, a nota somente é válida com o seu respectivo protocolo
     *
     * @param  string  $pathNFefile
     * @param  string  $pathProtfile
     * @param  boolean $saveFile
     * @return string
     * @throws Exception\RuntimeException
     */
    public function addProtocolo($xmlEnvio, $xmlProtocolo)
    {
        //carrega a NFe
        $docnfe = new Dom();
    
        $docnfe->loadXMLString($xmlEnvio);

        $nodenfe = $docnfe->getNode('NFe', 0);
        if ($nodenfe == '') {
            $msg = "O arquivo indicado como NFe não é um xml de NFe!";
            throw new \RuntimeException($msg);
        }
        if ($docnfe->getNode('Signature') == '') {
            $msg = "A NFe não está assinada!";
            throw new \RuntimeException($msg);
        }

        //carrega o protocolo
        $docprot = new Dom();

        $docprot->loadXMLString($xmlProtocolo);

        $nodeprots = $docprot->getElementsByTagName('protNFe');
        
        if ($nodeprots->length == 0) {
            $msg = "O arquivo indicado não contem um protocolo de autorização!";
            throw new \RuntimeException($msg);
        }
        //carrega dados da NFe
        $tpAmb = $docnfe->getNodeValue('tpAmb');
        $anomes = date(
            'Ym',
            $this->convertSefazTimeToTimestamp($docnfe->getNodeValue('dhEmi'))
        );
        $infNFe = $docnfe->getNode("infNFe", 0);
        $versao = $infNFe->getAttribute("versao");
        $chaveId = $infNFe->getAttribute("Id");
        $chaveNFe = preg_replace('/[^0-9]/', '', $chaveId);
        $digValueNFe = $docnfe->getNodeValue('DigestValue');
        //carrega os dados do protocolo
        for ($i = 0; $i < $nodeprots->length; $i++) {
            $nodeprot = $nodeprots->item($i);
            $protver = $nodeprot->getAttribute("versao");
            $chaveProt = $nodeprot->getElementsByTagName("chNFe")->item(0)->nodeValue;
            $digValueProt = ($nodeprot->getElementsByTagName("digVal")->length)
                ? $nodeprot->getElementsByTagName("digVal")->item(0)->nodeValue
                : '';
            $infProt = $nodeprot->getElementsByTagName("infProt")->item(0);
            if ($digValueNFe == $digValueProt && $chaveNFe == $chaveProt) {
                break;
            }
        }
        if ($digValueNFe != $digValueProt) {
            $msg = "Inconsistência! O DigestValue da NFe não combina com o do digVal do protocolo indicado!";
            throw new \RuntimeException($msg);
        }
        if ($chaveNFe != $chaveProt) {
            $msg = "O protocolo indicado pertence a outra NFe. Os números das chaves não combinam !";
            throw new \RuntimeException($msg);
        }
        //cria a NFe processada com a tag do protocolo
        $procnfe = new \DOMDocument('1.0', 'utf-8');
        $procnfe->formatOutput = false;
        $procnfe->preserveWhiteSpace = false;
        //cria a tag nfeProc
        $nfeProc = $procnfe->createElement('nfeProc');
        $procnfe->appendChild($nfeProc);
        //estabele o atributo de versão
        $nfeProcAtt1 = $nfeProc->appendChild($procnfe->createAttribute('versao'));
        $nfeProcAtt1->appendChild($procnfe->createTextNode($protver));
        //estabelece o atributo xmlns
        $nfeProcAtt2 = $nfeProc->appendChild($procnfe->createAttribute('xmlns'));
        $nfeProcAtt2->appendChild($procnfe->createTextNode('http://www.portalfiscal.inf.br/nfe'));
        //inclui a tag NFe
        $node = $procnfe->importNode($nodenfe, true);
        $nfeProc->appendChild($node);
        //cria tag protNFe
        $protNFe = $procnfe->createElement('protNFe');
        $nfeProc->appendChild($protNFe);
        //estabele o atributo de versão
        $protNFeAtt1 = $protNFe->appendChild($procnfe->createAttribute('versao'));
        $protNFeAtt1->appendChild($procnfe->createTextNode($versao));
        //cria tag infProt
        $nodep = $procnfe->importNode($infProt, true);
        $protNFe->appendChild($nodep);
        //salva o xml como string em uma variável
        $procXML = $procnfe->saveXML();
        //remove as informações indesejadas
        $procXML = $this->clearProt($procXML);
        
        return $procXML;
    }
    
        /**
     * convertSefazTimeToTimestamp
     * Converte a imformação de data e tempo contida na NFe
     *
     * @param  string $DH Informação de data e tempo extraida da NFe
     * @return timestamp UNIX Para uso com a funçao date do php
     */
    public function convertSefazTimeToTimestamp($dataHora = '')
    {
        if ($dataHora == '') {
            return '';
        }
        //inserido devido a casos de má formação do xml com
        //TZD +00:00 por exemplo
        $dataHora = str_replace('+', '-', $dataHora);
        $aDH = explode('T', $dataHora);
        $adDH = explode('-', $aDH[0]);
        $atDH = array('0','0','0');
        if (count($aDH) == 2) {
            $inter = explode('-', $aDH[1]);
            $atDH = explode(':', $inter[0]);
        }
        $timestamp = mktime($atDH[0], $atDH[1], $atDH[2], $adDH[1], $adDH[2], $adDH[0]);
        return $timestamp;
    }
    
    /**
     * clearProt
     * Limpa o xml após adição do protocolo
     *
     * @param  string $procXML
     * @return string
     */
    public function clearProt($procXML = '')
    {
        $procXML = self::clearMsg($procXML);
        $aApp = array('nfe','cte','mdfe');
        foreach ($aApp as $app) {
            $procXML = str_replace(
                'xmlns="http://www.portalfiscal.inf.br/'.$app.'" xmlns="http://www.w3.org/2000/09/xmldsig#"',
                'xmlns="http://www.portalfiscal.inf.br/'.$app.'"',
                $procXML
            );
        }
        return $procXML;
    }
    
    
    /**
     * clearMsg
     *
     * @param  string $msg
     * @return string
     */
    public static function clearMsg($msg)
    {
        $nmsg = str_replace(array(' standalone="no"','default:',':default',"\n","\r","\t"), '', $msg);
        $nnmsg = str_replace('> ', '>', $nmsg);
        if (strpos($nnmsg, '> ')) {
            $nnmsg = self::clearMsg((string) $nnmsg);
        }
        return $nnmsg;
    }
        
    public function assinaXml($xml) {
        return Signer::sign(
            $this->certificate,
            $xml,
            'infNFe'
        );
    }
     
    /**
     * getInfoSMTP
     * 
     * Função retorna dados de configuração SMTP
     * de cada Filial.
     *
     * @param  string $filial
     * @param  string $organizacao
     * @return Array
     */
    private function getInfoSMTP($filial, $organizacao)
    {
        /** @var $endefili Endefili */
        $endefili = Endefili::findOneByCdfilialAndIdtpendefiliAndNrorg(
            $filial,
            self::TIPO_END_PRINCIPAL,
            $organizacao
        );
        
        $password = $endefili->getCdsenhemailf()            ? $this->getDelphiCryptoUncripto($endefili->getCdsenhemailf(), false) : '@AppTecFood2015';
        $host     = $endefili->getDssmtpemailf()            ? explode(":" , $endefili->getDssmtpemailf() )[0] :  'smtp.teknisa.com';
        $username = $endefili->getDsloginsmtp()             ? $endefili->getDsloginsmtp()                     : 'app@tecfood.com';
        $secure   = $endefili->getIduttlsemail() == 'S'     ?  'tls'                                          :  ($endefili->getIdutsslemail() == 'S' ?  'ssl' : '');
        
        if ($endefili->getDssmtpemailf()){
            $port =   isset(explode(":" , $endefili->getDssmtpemailf())[1]) ? explode(":" , $endefili->getDssmtpemailf() )[1] :  '25' ;
        }else{
             $port = '25';
        }
        
        return array( 
            'host' => $host ,
            'username' => $username,
            'password'  => $password,
            'secure'    => $secure,
            'port'      => $port
        );
    }
    
    public static function getDelphiCryptoUncripto($password, $isEncrypt = true) {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar('2', XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema") // chave (salt)
        );
        //2
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUnCripto';
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );
            
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            return 'SoapError';
        }

        return $encryption;
    }
    
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
    
    public function aprovarNotasTransmissao ($notas, $response) {
        $temErroAprovar = false;
        
        try {
            foreach($notas as $nota) {
                $notaAprovar = self::getNotaFiscal($nota['NRACESSONFE']);
                $temErroAprovar = $this->aprovacaoAutomatica($notaAprovar, $temErroAprovar, $response);
            }
            
            if(!$temErroAprovar)   return array("SUCESSO_APROVACAO" => $this->sucessoAprovacao);
            else                   return array("ERROS_APROVACAO"   => $this->errosAprovacao);
            
        } catch (\Exception $e) {
            if($temErroAprovar) {
                return array("ERROS_APROVACAO" => $this->errosAprovacao);
            }
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function emailAutomaticoTransmissao ($notas, $config) {
        
        $config = $this->recolocarArrobaEmail(json_encode($config));
        $config = json_decode($config);
        $this->nfeTools = $this->getToolsNFe($config->ie, $config->cnpj, $config);
        
        $temErroEmail = false;
        try {
            foreach($notas as $nota) {
                $notaEmail = self::getNotaFiscal($nota['NRACESSONFE']);
                $temErroEmail = $this->enviarEmailProtocolo($notaEmail, $temErroEmail);
            }
            
            if(!$temErroEmail)   return array("SUCESSO_EMAIL" => $this->sucessoEmail);
            else                 return array("ERROS_EMAIL"   => $this->errosEmail);
            
        } catch (\Exception $e) {
            if($temErroEmail) {
                return array("ERROS_EMAIL" => $this->errosEmail);
            }
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}